//-------------------------------------------------------------------
//                                                                  
// Module      : KxoServletAgent.java                              
//                                                                  
// Description : CandleNet eBP JMS Outbound toolkit Servlet Helper  
//                                                                  
//-------------------------------------------------------------------
//                                                                  
// Copyright Candle Corporation 2001,2003                                
//                                                                  
//-------------------------------------------------------------------
//                                                                  
// Details:                                                         
//    Kxo toolkit Servlet helper class                             
// Warnings:                                                        
//    Creates a log file as                                         
//        \\Kxo_servlet.log   
//    for errors. 
//
// Diagnostics:
//  To add commentary logging rebuild with "logVerbose" set true  
//                                                                  
//                                                                  
//-------------------------------------------------------------------
//                                                                  
// Processing:                                                      
//                                                                  
// Agent instantiates Kxo User EJB and invokes method(s) on behalf 
// of servlet                                                       
//                                                                  
//-------------------------------------------------------------------
//                                                                  
// Return Value:                                                    
//    String from the invoked bean or human readable error text     
//                                                                  
//-------------------------------------------------------------------

package com.candle.roma.kxo.outbound.sample;

import com.candle.roma.kxo.LogHelper;

// JAVA Naming, RMI, EJB IO etc.
import javax.servlet.*;
import javax.servlet.http.*;

// these are so we can narrow the context
import javax.rmi.PortableRemoteObject;

// these are so we can use the context assoicated with the calling EJB
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;

import javax.ejb.*;
import java.util.Hashtable;

import java.io.*;
import java.util.Date;


//////////////////////////////////////////////////////////////////////
//
// CLASS: KxoServletAgent
//
/**
  * The KxoServletAgent class implements calling a User EJB method
  * call which is mapped via an Access Bean to the KxoEJB which
  * uses SOAP to invoke an eBP business service 
  * @author 	Candle
  * @version 1.0  $Revision: 1.5 $
  */
public class KxoServletAgent {

  // =======================================================================
  // Kxo-websphere integrated log helper wrapper
  private static final LogHelper LOG = new LogHelper(KxoServletAgent.class);

  // =======================================================================
  // The Kxo Outbound EJB Object - held as a static so that we only look
  // it up once
  private static com.candle.roma.kxo.outbound.sample.UserEJB my_ejb = null;

 /**
   * Instantiates Kxo User EJB 
   *
   */
  private synchronized void lookupUserBean() throws Exception
  {
    LOG.debug("lookupKxoBean(\"ejb/KxoOutboundHome\")"); 
    
    if ( my_ejb == null )
    {
      InitialContext ctx = new InitialContext();
      // Lookup based on our local resource reference which
      // was mapped to the EJB at deployment time
      LOG.debug("EJB lookup(\"ejb/UserEJBHome\")"); 
      Object obj = ctx.lookup("java:comp/env/ejb/UserEJBHome");

      com.candle.roma.kxo.outbound.sample.UserEJBHome li = (UserEJBHome)
               PortableRemoteObject.narrow(obj, UserEJBHome.class);

      my_ejb = li.create();

      LOG.info("Kxo Outbound Resolved EJB (\"ejb/UserEJBHome\")"); 
    }
    return;
  }

 /**
   * Instantiates Kxo User EJB and invokes method(s) on it and returns 
   * the results
   *
   * @param  requestInfo HttpServletRequest giving the input parms for the call
   * @param  result      StringBuffer contains the results of the method call
   * @since  1.0
   */
  public void run(HttpServletRequest requestInfo, StringBuffer result) {

    boolean doSet = false;

    try {
      LOG.debug("Kxo Outbound Servlet called");

      if ( my_ejb == null )
      {
        lookupUserBean();
      }

      //
      // If there is no parameter passed we send a Hard-wired message to 
      // the User EJB ...
      //
      String beanReqMsg;
      if (requestInfo == null)
      {
        // Set a "Product ID" number
        //
        beanReqMsg= "99999";
      }
      else
      {
        String msgPattern = requestInfo.getParameter("pattern");
        // Pattern is either message, request or datagram
        if (msgPattern.equals("message"))
        {
          // --------------------------------------------------------------
          String service = requestInfo.getParameter("firstname");
          String message = requestInfo.getParameter("lastname");
          LOG.debug("Kxo Outbound Servlet service and request message");
          result.append( "@KxoServletAgent:<p>" );
          result.append( my_ejb.serviceRequest(service, message) );
          result.append( "</p>" );
          LOG.debug("CEJO Servlet done."); 
          return;
        }
        // --------------------------------------------------------------
        LOG.debug("Kxo Outbound Servlet called for sample service");
        doSet = msgPattern.equals("datagram");
        String first = requestInfo.getParameter("firstname");
        String last = requestInfo.getParameter("lastname");
        String product = requestInfo.getParameter("productcode");

        LOG.debug("http Req paramters First[" + first +
                      "] Last[" + last +
                      "] ProductID[" + product + 
                      "] Type[" + msgPattern + "]" );

        if (product == null)
        {
          beanReqMsg= "11111";
          LOG.debug("Use hard product code 11111");
        }
        else
        {
          beanReqMsg = product;
        }
      }
      LOG.debug("Issue bean request as " + beanReqMsg); 

      result.append( "@KxoServletAgent[ProductId=" + beanReqMsg 
                     + "]<p>Message returned from UserEJB: <p>" );

      // ------------------------
      // Call a method on the EJB.
      // The result is appended to the "result" string which will be
      //  displayed by the browser. 
      // FYI: XML/SOAP tags won't be displayed unless they are wrapped in HTML.
      if ( doSet ) {
        LOG.debug("call buy() on User EJB"); 
        result.append( my_ejb.buy(beanReqMsg) );
      } else {
        LOG.debug("call inquire() on User EJB"); 
        result.append( my_ejb.inquire(beanReqMsg) );
      }
      LOG.debug("EJB call completed"); 
      

      // Client does not need to explicity remove the EJB - the container
      // will do it after the deployed session timetime
      // my_ejb.remove();
      LOG.debug("CEJO Servlet done."); 
    }
    catch (Exception e) {
      //
      // Exception occured - report
      //
      LOG.debug("Exception caught by KxoServletAgent.run", e);

      // It would not be normal practice to return details of the exception
      // to the user - just provide a generic "user readable" error and
      // suggested corrective actions.
      result.append("Error: Exception caught by KxoServlet. Detail: ");
      result.append(e.toString());
    }
  }
}
