//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoidir.cpp,v 1.15 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoidir.cpp,v $
// $Revision: 1.15 $
//--------------------------------------------------------------------
// The internal implementation of the RomaDirectoryItem and the
// RomaDirectoryAgent public interfaces.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoidir.hpp>
#include <kxcoipxy.hpp>
#include <kxcorda.hpp>
#include <kxcodefs.hpp>

TRACE_MODULE_VERSION($Revision: 1.15 $);


RomaImplAttrPair::RomaImplAttrPair(const char* n, const char* v)
{
    if (n == NULL)
    {
        n = "";
    }

    if (v == NULL)
    {
        v = "";
    }

    size_t nsize = strlen(n) + 1;
    size_t vsize = strlen(v) + 1;

    m_name = new char[nsize + vsize];
    memcpy(m_name, n, nsize);
    m_value = m_name + nsize;
    memcpy(m_value, v, vsize);
}

RomaImplAttrPair::~RomaImplAttrPair()
{
    delete[] m_name;
}

//====================================================================
// The RomaImplDirItem class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
// All class members will be set with the default values.
//--------------------------------------------------------------------
RomaImplDirItem::RomaImplDirItem()
    : m_dirty(true), m_option(Roma::FORCE), m_dn(NULL)
{
    TRACE_ENTRY(RomaImplDirItem::RomaImplDirItem, << TRACE(this));

    m_buffer.Status = KXCBUFSTAT_USER;
    m_buffer.pBuffer = NULL;
    m_buffer.BufLen = 0;
    m_size = kxcRoundLength(sizeof(KXCDIRITEMMAINHDR));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the item with the specified distinguished name.
//--------------------------------------------------------------------
RomaImplDirItem::RomaImplDirItem(const char* dn, const char* objClass)
    : m_dirty(true), m_option(Roma::FORCE), m_dn(NULL)
{
    TRACE_ENTRY(RomaImplDirItem::RomaImplDirItem, << TRACE(this)
                                                  << TRACE(dn)
                                                  << TRACE(objClass));

    m_buffer.Status = KXCBUFSTAT_USER;
    m_buffer.pBuffer = NULL;
    m_buffer.BufLen = 0;
    m_size = kxcRoundLength(sizeof(KXCDIRITEMMAINHDR));
    setDN(dn);
    addAttribute("objectclass", objClass);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// The RomaImplDirAgent uses this constructor to create an item from
// the C API item data, which is returned on a directory lookup.
// All items in the directory have an "objectclass" attribute, so
// the retrieved item will have an objectclass by definition.
//--------------------------------------------------------------------
RomaImplDirItem::RomaImplDirItem(const KXCBUFFER* buf)
    : m_dirty(true), m_option(Roma::FORCE), m_dn(NULL), m_size(0)
{
    TRACE_ENTRY(RomaImplDirItem::RomaImplDirItem, << TRACE(this)
                                                  << TRACE(buf));

    // Initialize the KXCBUFFER.
    m_buffer.Status = KXCBUFSTAT_USER;
    m_buffer.pBuffer = NULL;
    m_buffer.BufLen = 0;

    // Although this object is constructed from a valid KXCBUFFER, it
    // is unlikely that it will be needed again. Therefore it is not
    // stored in its raw form and we set m_dirty to indicate that it
    // would need to be rebuilt on-demand. The total size is stored
    // so that an appropriately sized buffer can be allocated if it
    // is required. See the asBuffer() method.

    // Set sensible initial values and get the number of attributes
    // from the specified item data.
    KXCBOOL status = 0;
    KXCULONG number = 0;
    kxcDirCountAttributes(buf, number, status);

    if (!status)
    {
        // The buffer pointer is NULL.
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_VOID_RETURN;
    }

    // Remember the buffer size.
    m_size = buf->BufLen;

    if (!number) 
    {
        // There are no attributes, so nothing to do further.
        TRACE_WARNING(KXCOM_NO_ATTRS);
        TRACE_VOID_RETURN;
    }

    // At this point we know how many attributes are present.
    // Now pick out the distinguished name.
    KXCCHAR* dn = NULL;
    kxcDirGetDn(buf, dn, status);

    if (!status) 
    {
        // The buffer pointer is NULL.
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_VOID_RETURN;
    }

    // Set the distinguished name of the item.
    setDN((const char*)dn);

    // Go through the buffer and create corresponding list of pairs.
    // We could just rely on the copy of the item data buffer and
    // access it as required using kxcDirGetAttributeAt().
    // However, the list is almost essential when an application
    // builds up its own RomaDirectoryItem, and it is a more flexible
    // approach.
    for (KXCULONG idx = 0; idx < number; ++idx) 
    {
        // Extract a name and a value of the item located at the
        // index.
        KXCCHAR* itemName;
        KXCCHAR* itemValue;
        kxcDirGetAttributeAt(buf, idx, itemName, itemValue, status);

        if (!status) 
        {
            // Error.
            TRACE_WARNING(KXCOM_DIRATTR_EXTRACT_FAILED(idx));
            break;
        }

        // Create the RomaImplAttrPair object from the exctracted
        // name and value.
        RomaImplAttrPair* pair = new RomaImplAttrPair(
            (const char*)itemName, (const char*)itemValue);

        if (!pair)
        {
            // Out of memory.
            TRACE_WARNING(KXCOM_NEW_FAILED);
            break;
        }

        // Add the pair to the attributes list.
        m_pairs.Append(pair);
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplDirItem::~RomaImplDirItem()
{
    TRACE_ENTRY(RomaImplDirItem::~RomaImplDirItem, << TRACE(this));

    // Free dynamically allocated memory.
    clear();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// This method removes content of the item.
//--------------------------------------------------------------------
void RomaImplDirItem::clear()
{
    TRACE_ENTRY(RomaImplDirItem::clear, << TRACE(this));

    delete[] m_dn;
    m_dn = NULL;

    delete[] (char*)m_buffer.pBuffer;
    m_buffer.pBuffer = NULL;
    m_buffer.BufLen = 0;
    m_size = kxcRoundLength(sizeof(KXCDIRITEMMAINHDR));

    // Remove attributes from the list.
    RomaImplAttrPairLListIterator it(m_pairs);
    RomaImplAttrPair* pair;

    while (it.GetNext(&pair)) 
    {
        delete pair;
    }

    m_pairs.Clear();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set the distinguished name.
//--------------------------------------------------------------------
void RomaImplDirItem::setDN(const char* dn)
{
    TRACE_ENTRY(RomaImplDirItem::setDN, << TRACE(this) << TRACE(dn));

    // Check the specified pointer.
    if (!dn)
    {
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_VOID_RETURN;
    }

    // Check the string is not empty.
    if (*dn == '\0')
    {
        TRACE_WARNING(KXCOM_EMPTY_STRING);
        TRACE_VOID_RETURN;
    }

    // If there is a distinguished name already, delete it.
    if (m_dn) 
    {
        m_size -= strlen(m_dn) + 1;
        delete[] m_dn;
        m_dn = NULL;
    }

    // Allocate memory for the specified distinguished name.
    size_t len = strlen(dn) + 1;
    m_dn = new char[len];

    if (m_dn)
    {
        // Copy the specified distinguished name.
        memcpy(m_dn, dn, len);

        // Recalculate item data size.
        m_size += kxcRoundLength(sizeof(KXCULONG));
        m_size += kxcRoundLength(len);

        // Mark the item as dirty.
        m_dirty = true;
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Add the attribute to the item.
//--------------------------------------------------------------------
void RomaImplDirItem::addAttribute(const char* name, const char* value)
{
    TRACE_ENTRY(RomaImplDirItem::addAttribute, << TRACE(this) 
                                               << TRACE(name)
                                               << TRACE(value));

    // Check that specified pointers are OK.
    if (!name || !value)
    {
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_VOID_RETURN;
    }

    // Create the RomaImplAttrPair from the specified name and value.
    RomaImplAttrPair* pair = new RomaImplAttrPair(name, value);

    if (pair)
    {
        // Add the pair to the list of attributes.
        m_pairs.Append(pair);

        // Mark the item as dirty.
        m_dirty = true;

        // Recalculate the item data size. The way used by the C API
        // to extract the item from the buffer can cause the
        // "unaligned memory access" error. Thus, we need to round
        // the data buffer size to word boundary using the
        // kxcRoundLength() macro.
        m_size += kxcRoundLength(sizeof(KXCDIRITEMHDR));
        size_t len = strlen(name) + 1;
        m_size += kxcRoundLength(len);
        len = strlen(value) + 1;
        m_size += kxcRoundLength(len);
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_NEW_FAILED);
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Return a KXCBUFFER containing the header, distinguished name
// and an array of attributes (name/value pairs).
// NOTE: the caller should not free the buffer we pass back.
//--------------------------------------------------------------------
const KXCBUFFER* RomaImplDirItem::asBuffer()
{
    TRACE_ENTRY(RomaImplDirItem::asBuffer, << TRACE(this));

    if (!m_dirty)
    {
        // The cached C representation of the item is up-to-date,
        // so we just return it.
        TRACE_EVENT(KXCOM_NOT_DIRTY);
        TRACE_INT_RETURN(&m_buffer);
    }

    // Otherwise, we need to rebuild the buffer. First delete an
    // existing buffer.
    delete[] (char*)m_buffer.pBuffer;
    m_buffer.BufLen = 0;

    // We now have a valid KXCBUFFER without a buffer. As attributes
    // were added to this object, m_size was incremented to track the
    // total size requirements for the resulting buffer. Now we can
    // allocate a buffer of the correct size.

    // Make up total size to next word boundary.
    m_size = kxcRoundLength(m_size);
    m_buffer.pBuffer = new char[m_size];

    if (!m_buffer.pBuffer)
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(m_size));
        TRACE_INT_RETURN(NULL);
    }

    // Put the distinguished name to the buffer.
    KXCBOOL success;
    m_buffer.BufLen = m_size;
    memset(m_buffer.pBuffer, 0, m_size);
    kxcSetDirDn(&m_buffer, m_dn, success);

    if (!success) 
    {
        // Parameters invalid, or the buffer is not large enough.
        delete[] (char*)m_buffer.pBuffer;
        m_buffer.pBuffer = NULL;
        m_buffer.BufLen = 0;
        TRACE_INT_RETURN(NULL);
    }

    // We walk through the attributes list, adding each attribute in
    // turn to the buffer.
    RomaImplAttrPairLListIterator it(m_pairs);
    RomaImplAttrPair* pair;

    while (it.GetNext(&pair)) 
    {
        kxcSetDirItem(&m_buffer, pair->getName(),
            pair->getValue(), success);

        if (!success) 
        {
            // Parameters invalid, or the buffer is not large enough.
            delete[] (char*)m_buffer.pBuffer;
            m_buffer.pBuffer = NULL;
            m_buffer.BufLen = 0;
            TRACE_INT_RETURN(NULL);
        }
    }

    // Now the cached item data is up-to-date.
    m_dirty = false;
    TRACE_INT_RETURN(&m_buffer);
}

//--------------------------------------------------------------------
// Get a name of an attribute located at the specified index.
//--------------------------------------------------------------------
const char* RomaImplDirItem::getAttrName(int index) const
{
    TRACE_ENTRY(RomaImplDirItem::getAttrName, << TRACE(this)
                                              << TRACE(index));

    RomaImplAttrPair* pair = NULL;
    const char* name = NULL;

    // Get a RomaImplAttrPair at the index.
    // If the index is invalid, the pair will be set to NULL.
    if (m_pairs.GetAtIndex(index, &pair) && pair)
    {
        name = pair->getName();
    }

    TRACE_RETURN(name);
}

//--------------------------------------------------------------------
// Get a value of an attribute located at the specified index.
//--------------------------------------------------------------------
const char* RomaImplDirItem::getAttrValue(int index) const
{
    TRACE_ENTRY(RomaImplDirItem::getAttrValue, << TRACE(this)
                                               << TRACE(index));

    RomaImplAttrPair* pair = NULL;
    const char* value = NULL;

    // Get a RomaImplAttrPair at the index.
    // If the index is invalid, the pair will be set to NULL.
    if (m_pairs.GetAtIndex(index, &pair) && pair)
    {
        value = pair->getValue();
    }

    TRACE_RETURN(value);
}

//--------------------------------------------------------------------
// Get a value of an attribute having the specified name and occuring
// the specified number of times.
//--------------------------------------------------------------------
const char* RomaImplDirItem::getAttrValue(const char* name, int index) const
{
    TRACE_ENTRY(RomaImplDirItem::getAttrValue, << TRACE(this)
                                               << TRACE(name)
                                               << TRACE(index));

    if (!name)
    {
        // Invalid pointer specified.
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_INT_RETURN(NULL);
    }

    RomaImplAttrPair* pair;
    RomaImplAttrPairLListIterator it(m_pairs);
    int seen = 0;
    const char* value = NULL;

    // Step through the list, check for the specified name. If the
    // name matches, increment the occurrence count and return the
    // entry if we reached the specified occurrence.
    while (it.GetNext(&pair)) 
    {
        if (!strcmp(pair->getName(), name) && (seen++ == index))
        {
            value = pair->getValue();
            break;
        }
    }

    TRACE_RETURN(value);
}

//--------------------------------------------------------------------
// Indicates whether the RomaDirectoryAgent::storeItem() should add
// or replace an item.
//--------------------------------------------------------------------
void RomaImplDirItem::setMustExist(bool b)
{
    m_option = b ? Roma::MUST_EXIST : Roma::MUST_NOT_EXIST;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplDirItem::trace(RomaTraceStream& s) const
{
    s << TRACE(m_size) << TRACE(m_dn) << TRACE(m_option)
      << TRACE(m_dirty);

    RomaImplAttrPair* pair;
    RomaImplAttrPairLListIterator it(m_pairs);

    while (it.GetNext(&pair)) 
    {
        s << "[" << pair->getName() << ", " << pair->getValue() << "]\n";
    }
}


//====================================================================
// The RomaImplDirAgent class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
// Sets the default values to all data members.
//--------------------------------------------------------------------
RomaImplDirAgent::RomaImplDirAgent()
    : m_modCount(0), m_context(NULL)
{
    TRACE_ENTRY(RomaImplDirAgent::RomaImplDirAgent, << TRACE(this));

    KXCDIRSEARCHREQ req = { KXCDIRSEARCHREQ_DEFAULT };
    m_request = req;

    KXCDIRSEARCHRES res = { KXCDIRSEARCHRES_DEFAULT };
    m_result = res;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
// Releases dynamilcally allocated memory and the shared context
// instance.
//--------------------------------------------------------------------
RomaImplDirAgent::~RomaImplDirAgent()
{
    TRACE_ENTRY(RomaImplDirAgent::~RomaImplDirAgent, << TRACE(this));

    delete[] (char*)m_request.StartingPoint;
    delete[] (char*)m_request.SearchName;

    if (m_context)
    {
        m_context->releaseInstance();
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set parameters for the directory items lookup using the specified
// context. The context will be locked until the next lookup or the
// object destruction. The actual lookup will be performed later (see
// the getMatches() method).
//--------------------------------------------------------------------
bool RomaImplDirAgent::lookup(RomaImplContext& context,
                              const char* origin,
	                          const char* name,
	                          Roma::SearchType type,
	                          Roma::SearchOption option)
{
    TRACE_ENTRY(RomaImplDirAgent::lookup, << TRACE(this)
                                          << TRACE(origin)
                                          << TRACE(name)
                                          << TRACE(type)
                                          << TRACE(option)
                                          << TRACE(context));

    // Check the specified pointers.
    if (!origin || !name)
    {
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_INT_RETURN(false);
    }

    // Check that the context is connected.
    if (!context.checkConnected(KXCOCALLTYPE_DIRLOOKUP))
    {
        TRACE_INT_RETURN(false);
    }

    // If there is a context already, release it.
    if (m_context)
    {
        m_context->releaseInstance();
    }

    // Capture the specified context.
    m_context = &context;
    m_context->lockInstance();

    // Setup the KXCDIRSEARCHREQ structure with the specified
    // parameters.
    size_t len = strlen(origin) + 1;
    delete[] (char*)m_request.StartingPoint;
    m_request.StartingPoint = (PKXCCHAR)new char[len];

    if (!m_request.StartingPoint)
    {
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
        TRACE_INT_RETURN(false);
    }

    memcpy(m_request.StartingPoint, origin, len);

    len = strlen(name) + 1;
    delete[] (char*)m_request.SearchName;
    m_request.SearchName = (PKXCCHAR)new char[len];

    if (!m_request.SearchName) 
    {
        delete[] (char*)m_request.StartingPoint;
        m_request.StartingPoint = NULL;
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(len));
        TRACE_INT_RETURN(false);
    }

    memcpy(m_request.SearchName, name, len);
    m_request.SearchType = type;
    m_request.Options = option;

    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Get a number of found items.
//--------------------------------------------------------------------
int RomaImplDirAgent::getMatches()
{
    bool tmp;
    return getMatches(tmp);
}

//--------------------------------------------------------------------
// Get a number of found items and the modification indicator.
//--------------------------------------------------------------------
int RomaImplDirAgent::getMatches(bool& modified)
{
    TRACE_ENTRY(RomaImplDirAgent::getMatches, << TRACE(this));

    modified = false;

    // Check that the object is perpared for a lookup, i.e. the
    // lookup() method was called and we have captured a context.
    if (!m_context)
    {
        TRACE_WARNING(KXCOM_NO_CONTEXT);
        TRACE_INT_RETURN(0);
    }

    // Check that the context is connected.
    if (!m_context->checkConnected(KXCOCALLTYPE_DIRLOOKUP))
    {
        TRACE_INT_RETURN(0);
    }

    int matches = 0;

    // Lookup the directory using the parameters which have been
    // specified for the lookup() method.
    if (kxcDirLookUp((HKXC)(*m_context), &m_request, &m_result)) 
    {
        matches = m_result.Matches;

        // If the directory modification counters do not match,
        // then the directory was modified since the last lookup.
        if (m_result.ModCount != m_modCount) 
        {
            m_modCount = m_result.ModCount;
            modified = true;
        }
    }

    TRACE_INT_RETURN(matches);
}

//--------------------------------------------------------------------
// Get an item located at the specified index.
//--------------------------------------------------------------------
RomaDirectoryItem* RomaImplDirAgent::getItem(int i)
{
    TRACE_ENTRY(RomaImplDirAgent::getItem, << TRACE(this) << TRACE(i));

    // Check that the object is perpared for a lookup, i.e. the
    // lookup() method was called and we have captured a context.
    if (!m_context)
    {
        TRACE_WARNING(KXCOM_NO_CONTEXT);
        TRACE_INT_RETURN(NULL);
    }

    // Check that the context is connected.
    if (!m_context->checkConnected(KXCOCALLTYPE_DIRGETITEM))
    {
        TRACE_INT_RETURN(NULL);
    }

    // Check that there are matching items.
    if (!getMatches())
    {
        TRACE_WARNING(KXCOM_NO_MATCHES);
        TRACE_INT_RETURN(NULL);
    }

    // Get the item into the buffer, create and return the
    // corresponding C++ object from the C representation of the item.
    HKXC h = (HKXC)(*m_context);
    RomaDirectoryItem* res = NULL;
    KXCBUFFER data = { KXCBUFFER_DEFAULT };
    KXCDIRITEMREQ item = { KXCDIRITEMREQ_DEFAULT };
    item.EntryNumber = i;

    if (kxcDirGetItem(h, &item, &data))
    {
        RomaImplDirItem* impl = new RomaImplDirItem(&data);
        kxcFreeBuffer(NULL_HKXC, &data);
        res = RomaImplProxy::createPublic(impl);
    }

    TRACE_INT_RETURN(res);
}

//--------------------------------------------------------------------
// Get an item located at the specified index. Also get the
// modification indicator.
//--------------------------------------------------------------------
RomaDirectoryItem* RomaImplDirAgent::getItem(int i, bool& modified)
{
    TRACE_ENTRY(RomaImplDirAgent::getItem, << TRACE(this));

    modified = false;

    // Check that the object is perpared for a lookup, i.e. the
    // lookup() method was called and we have captured a context.
    if (!m_context)
    {
        TRACE_WARNING(KXCOM_NO_CONTEXT);
        TRACE_INT_RETURN(NULL);
    }

    // Check that the context is connected.
    if (!m_context->checkConnected(KXCOCALLTYPE_DIRGETITEM))
    {
        TRACE_INT_RETURN(NULL);
    }

    // Determine whether the directory was modified since
    // the last lookup.
    KXCULONG modCount;
    KXCBOOL status = kxcDirGetModCount((HKXC)(*m_context), &modCount);

    if (status == FALSE)
    {
        TRACE_WARNING(<< "Failed to get modification count.");
        TRACE_INT_RETURN(NULL);
    }

    if (modCount != m_modCount) 
    {
        m_modCount = modCount;
        modified = true;
    }

    // Return the item found at the index.
    RomaDirectoryItem* res = getItem(i);
    TRACE_INT_RETURN(res);
}

//--------------------------------------------------------------------
// Delete an item.
// An item identified by the specified distinguished name will be
// removed from the directory. This operation requires an
// autorization, so an application should specify a valid user id and
// password or use the RomaDirectoryAccess object to provide
// authentication information for the connect() method. In the latter
// case, the user id and password parameters may be NULL.
//--------------------------------------------------------------------
bool RomaImplDirAgent::deleteItem(const char* dn, 
                                  const char* id,
			                      const char* pw)
{
    TRACE_ENTRY(RomaImplDirAgent::deleteItem, << TRACE(this)
                                              << TRACE(dn)
                                              << TRACE(id)
                                              << TRACE(pw));

    // Check that the object is perpared for a lookup, i.e. the
    // lookup() method was called and we have captured a context.
    if (!m_context)
    {
        TRACE_WARNING(KXCOM_NO_CONTEXT);
        TRACE_INT_RETURN(false);
    }

    // Check that the context is connected.
    if (!m_context->checkConnected(KXCOCALLTYPE_DIRDELETEITEM))
    {
        TRACE_INT_RETURN(false);
    }

    // The distinguished name is required parameter.
    if (!dn)
    {
        TRACE_WARNING(KXCOM_NULL_POINTER);
        TRACE_INT_RETURN(false);
    }

    // Force the directory lookup.
    getMatches();

    // Calculate the size of memory required to store the
    // distinguished name and the KXCDIRITEMMAINHDR structure.
    // The size must be rounded in order to avoid the
    // "misaligned memory access" crash in the C API.
    size_t s = sizeof(KXCDIRITEMMAINHDR) + sizeof(KXCULONG);
    size_t length = kxcRoundLength(s);
    s = strlen(dn) + 1;
    length += kxcRoundLength(s);

    KXCBUFFER data;
    data.Status = KXCBUFSTAT_USER;
    data.BufLen = length;
    data.pBuffer = new char[length];

    if (!data.pBuffer)
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(length));
        TRACE_INT_RETURN(false);
    }

    memset(data.pBuffer, 0, length);

    // Pack the distinguished name into the buffer.
    kxcDirSetDnEx(&data, (PKXCCHAR)dn);

    // Setup the KXCDIRUPDATEOPT structure with the user id and
    // the password (if specified).
    KXCDIRUPDATEOPT opt = { KXCDIRUPDATEOPT_DEFAULT };

    if (id)
    {
        strncpy((char*)opt.Id, id, KXCOBJECT_NAME_LENGTH - 1);
        opt.Id[KXCOBJECT_NAME_LENGTH - 1] = 0;
    }

    if (pw)
    {
        strncpy((char*)opt.Password, pw, KXCOBJECT_NAME_LENGTH - 1);
        opt.Password[KXCOBJECT_NAME_LENGTH - 1] = 0;
    }

    // Delete the item.
    bool success = 
        (kxcDirDeleteItem((HKXC)(*m_context), &opt, &data) == TRUE);

    // Free memory allocated for the buffer.
    delete[] (char*)data.pBuffer;

    // If there is an error, gather and save information about it.
    m_context->setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Store an item.
// This operation requires an autorization, so an application should
// specify a valid user id and password or use the RomaDirectoryAccess
// object to provide authentication information for the connect()
// method. In the latter case, the user id and password parameters may
// be NULL.
//--------------------------------------------------------------------
bool RomaImplDirAgent::storeItem(const RomaDirectoryItem& item,
			                     const char* id,
			                     const char* pw)
{
    TRACE_ENTRY(RomaImplDirAgent::storeItem, << TRACE(this)
                                             << TRACE(item)
                                             << TRACE(id)
                                             << TRACE(pw));

    // Check that the object is perpared for a lookup, i.e. the
    // lookup() method was called and we have captured a context.
    if (!m_context)
    {
        TRACE_WARNING(KXCOM_NO_CONTEXT);
        TRACE_INT_RETURN(false);
    }

    // Check that the context is connected.
    if (!m_context->checkConnected(KXCOCALLTYPE_DIRSTOREITEM))
    {
        TRACE_INT_RETURN(false);
    }

    // Setup the KXCDIRUPDATEOPT structure with the item's store
    // options, the user id and the password (if specified).
    KXCDIRUPDATEOPT opt = { KXCDIRUPDATEOPT_DEFAULT };
    opt.Options = RomaImplProxy::getImpl(item).getOption();

    if (id)
    {
        strncpy((char*)opt.Id, id, KXCOBJECT_NAME_LENGTH - 1);
        opt.Id[KXCOBJECT_NAME_LENGTH - 1] = 0;
    }

    if (pw)
    {
        strncpy((char*)opt.Password, pw, KXCOBJECT_NAME_LENGTH - 1);
        opt.Password[KXCOBJECT_NAME_LENGTH - 1] = 0;
    }

    // Get the KXCBUFFER containing the C API representation of the
    // item data and update the directory.
    RomaImplDirItem* impl = 
        (RomaImplDirItem*)&RomaImplProxy::getImpl(item);
    bool success = (kxcDirUpdateItem((HKXC)(*m_context), &opt, 
        (PKXCBUFFER)impl->asBuffer()) == TRUE);

    // If there is an error, gather and save information about it.
    m_context->setErrorStatus(success);
    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplDirAgent::trace(RomaTraceStream& s) const
{
    s << TRACE(m_context) << TRACE(m_modCount)
      << TRACE((const void*)m_request.StartingPoint)
      << TRACE((const void*)m_request.SearchName)
      << TRACE(m_request.SearchType) << TRACE(m_request.Options)
      << TRACE(m_result.Matches) << TRACE(m_result.ModCount);
}

