//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoidir.hpp,v 1.13 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoidir.hpp,v $
// $Revision: 1.13 $
//--------------------------------------------------------------------
// The internal implementation of the RomaDirectoryItem and 
// the RomaDirectoryAgent public interfaces.
//--------------------------------------------------------------------

#ifndef __KXCOIDIR_HPP__
#define __KXCOIDIR_HPP__

#include "kxcotrb.hpp"
#include "kxcoutil.hpp"
#include "kxcoicxt.hpp"


//--------------------------------------------------------------------
// Forward declarations.
//--------------------------------------------------------------------
class RomaDirectoryItem;


//--------------------------------------------------------------------
// The RomaImplAttrPair class.
// 
// This class represents a directory item attribute. It is a simple
// name/value pair.
//--------------------------------------------------------------------
class RomaImplAttrPair
{
public:
    //----------------------------------------------------------------
    // Initializing constructor.
    //----------------------------------------------------------------
    RomaImplAttrPair(const char* n, const char* v);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplAttrPair();

    const char* getName() { return m_name; }
    const char* getValue() { return m_value; }

private:
    //----------------------------------------------------------------
    // Members.
    //----------------------------------------------------------------
    char* m_name;
    char* m_value;

    //----------------------------------------------------------------
    // Not implemented.
    //----------------------------------------------------------------
    RomaImplAttrPair(const RomaImplAttrPair&);
    RomaImplAttrPair& operator = (const RomaImplAttrPair&);
};


//--------------------------------------------------------------------
// Declaration of the linked list of the RomaImplAttrPair objects.
//--------------------------------------------------------------------
KLINKED_LIST_OF(RomaImplAttrPair)


//--------------------------------------------------------------------
// The RomaImplDirItem class.
//
// This class is the internal implementation of the RomaDirectoryItem
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplDirItem : public RomaTraceable,
                                public RomaImplShareable 
{
public:
    //----------------------------------------------------------------
    // Constructors.
    //----------------------------------------------------------------
    RomaImplDirItem();
    RomaImplDirItem(const char*, const char*);
    RomaImplDirItem(const KXCBUFFER*);

    //----------------------------------------------------------------
    // Set/get the distinguished name.
    //----------------------------------------------------------------
    void setDN(const char*);
    const char* getDN() const { return m_dn; }

    //----------------------------------------------------------------
    // Get the number of attributes.
    //----------------------------------------------------------------
    int getNumber() const { return m_pairs.Length(); }

    //----------------------------------------------------------------
    // Get the store options.
    //----------------------------------------------------------------
    Roma::StoreOption getOption() const { return m_option; }

    //----------------------------------------------------------------
    // Add the attribute to the item.
    //----------------------------------------------------------------
    void addAttribute(const char*, const char*);

    //----------------------------------------------------------------
    // Get a name of the N'th attribute, where N is zero-based index.
    //----------------------------------------------------------------
    const char* getAttrName(int) const;

    //----------------------------------------------------------------
    // Get a value of the N'th attribute, where N is zero-based index.
    //----------------------------------------------------------------
    const char* getAttrValue(int) const;

    //----------------------------------------------------------------
    // Get a value of the N'th attribute, where N is zero-based index
    // in the set of the attributes having the specified name.
    //----------------------------------------------------------------
    const char* getAttrValue(const char*, int = 0) const;

    //----------------------------------------------------------------
    // If set to true, it is allowed to replace the item during the
    // RomaDirectoryAgent::storeItem() call, but not add it.
    //----------------------------------------------------------------
    void setMustExist(bool);

    //----------------------------------------------------------------
    // Convert item data into the C API representation.
    //----------------------------------------------------------------
    const KXCBUFFER* asBuffer();

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplDirItem();

    //----------------------------------------------------------------
    // Removes all item data.
    //----------------------------------------------------------------
    void clear();

    //----------------------------------------------------------------
    // The list of item attributes.
    //----------------------------------------------------------------
    RomaImplAttrPairLList m_pairs;

    //----------------------------------------------------------------
    // The store options.
    //----------------------------------------------------------------
    Roma::StoreOption m_option;

    //----------------------------------------------------------------
    // The cached C API representation of the item.
    //----------------------------------------------------------------
    KXCBUFFER m_buffer;

    //----------------------------------------------------------------
    // The distinguished name of the item.
    //----------------------------------------------------------------
    char* m_dn;

    //----------------------------------------------------------------
    // Indicates whether the cached C API representation is dirty.
    //----------------------------------------------------------------
    bool m_dirty;

    //----------------------------------------------------------------
    // The size of the cached C API representation.
    //----------------------------------------------------------------
    int m_size;

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplDirItem(const RomaImplDirItem&);
    RomaImplDirItem& operator = (const RomaImplDirItem&);

    friend class KxcoStreamer;
};


//--------------------------------------------------------------------
// The RomaImplDirAgent class.
//
// This class is the internal implementation of the RomaDirectoryAgent
// public interface.
//--------------------------------------------------------------------
class KXCODLL RomaImplDirAgent : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaImplDirAgent();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    ~RomaImplDirAgent();

    //----------------------------------------------------------------
    // Search items in the directory.
    //----------------------------------------------------------------
    bool lookup(RomaImplContext&, const char*, const char*,
                Roma::SearchType, Roma::SearchOption);

    //----------------------------------------------------------------
    // Get a number of found items.
    //----------------------------------------------------------------
    int getMatches();
    int getMatches(bool&);

    //----------------------------------------------------------------
    // Get an item by the zero-based index.
    //----------------------------------------------------------------
    RomaDirectoryItem* getItem(int);
    RomaDirectoryItem* getItem(int, bool&);

    //----------------------------------------------------------------
    // Detete the item.
    //----------------------------------------------------------------
    bool deleteItem(const char*, const char*, const char*);

    //----------------------------------------------------------------
    // Store the item.
    //----------------------------------------------------------------
    bool storeItem(const RomaDirectoryItem&, const char*, const char*);

    //----------------------------------------------------------------
    // Output trace information.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;

private:
    //----------------------------------------------------------------
    // Pointer to the shared context.
    //----------------------------------------------------------------
    RomaImplContext* m_context;

    //----------------------------------------------------------------
    // The directory lookup request.
    //----------------------------------------------------------------
    KXCDIRSEARCHREQ m_request;

    //----------------------------------------------------------------
    // The result of the directory lookup.
    //----------------------------------------------------------------
    KXCDIRSEARCHRES m_result;

    //----------------------------------------------------------------
    // The directory modifications counter. Used to determine whether
    // the directory was modified since the last lookup.
    //----------------------------------------------------------------
    KXCULONG m_modCount;

    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaImplDirAgent(const RomaImplDirAgent&);
    RomaImplDirAgent& operator = (const RomaImplDirAgent&);
};


#endif // __KXCOIDIR_HPP__

