//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaException.java
//
// Defines      : class RomaException
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* This class introduces Roma errors. It completely reflects the
* structure of error information Roma API.
*
* The get methods are public, for use by application.
* The set functions have package visibility for use
* by RomaExceptionMessage
* 
*  @see RomaExceptionMessage
*  @see RomaErrorInformation
*/
class RomaException extends Exception implements Roma
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  /**
  * Constructs a new RomaException with no additional information.
  */
  public RomaException()
  {
  }
  
  /**
  * Constructs a new RomaException object with a given textual
  * description.
  */
  public RomaException( String s )
  {
    super( s );
  }

  /**
  * Non-public constructor that takes an existing KxcError object 
  * and sets the members of this new instance according to the
  * state of that object.
  */
  RomaException( KxcError err )
  {
    super( createErrorText( err ) );
    m_status = err.status();
    
    if ( hasPrimaryInfo() )
      m_primInfo = new RomaErrorInformation( err.result() );
    else
      m_primInfo = new RomaErrorInformation();
    
    if ( hasSecondaryInfo() )
      m_secInfo = new RomaErrorInformation( err.subResult() );
    else
      m_secInfo = new RomaErrorInformation();
  }
  
  /**
  * Returns true if error status() != STATUS_NONE
  */
  public boolean anyError()
  {
    return m_status != STATUS_NONE;
  }
  
  /**
  * Returns either the completion code of the error or else
  * the code Roma.CC_OK if no error information is available.
  *
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public int compCode( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.compCode();		
      else
        return m_secInfo.compCode();	
    }
    else
      return Roma.CC_OK;
  }
  
  /**
  * Returns the return code
  *
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public int returnCode( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.returnCode();		
      else
        return m_secInfo.returnCode();	
    }
    else
      return 0;
  }

  /**
  * Returns either the reason code of the error or else 0
  * if no error information is available.
  *	
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public int reason( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.reason();		
      else
        return m_secInfo.reason();	
    }
    else
      return 0;
  }
  
  /**
  * Returns either the error type code of the error or else 0
  * if no error information is available.
  *	
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public int errorType( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.type();		
      else
        return m_secInfo.type();	
    }
    else
      return 0;
  }
  
  /**
  * If an error occurred, returns the operation type of the API
  * call. Otherwise returns 0.
  *	
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public int operationType( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.operationType();		
      else
        return m_secInfo.operationType();	
    }
    else
      return 0;
  }
  
  /**
  * Returns the position if an error has occurred. Otherwise 
  * returns the value 0.
  *
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public int position( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.position();		
      else
        return m_secInfo.position();	
    }
    else
      return 0;
  }
  
  /**
  * Returns the additional information if an error has occurred. Otherwise 
  * returns an empty string.
  *
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  */
  public String additionalInfo( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.additionalInfo();		
      else
        return m_secInfo.additionalInfo();	
    }
    else
      return "";
  }
  
  /**
  * Returns the error string if an error has occurred. Otherwise 
  * returns an empty string.
  *
  * @param errorSubclass Kxc.PRIMARY_INFO   - set primary info.
  *	                     Kxc.SECONDARY_INFO - set secondary info.
  *
  * @param errorFormat   Kxc.ERROR_TEXT_FULL    
  *					             Kxc.ERROR_TEXT_COMPACT
  *
  */
  public String errorText(int errorSubclass, 
                          int errorFormat)
  {
    if ( anyError() )
    {
      RomaErrorInformation info;
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        info = m_primInfo;
      else
        info = m_secInfo;
      
      if ( errorFormat == Kxc.ERROR_TEXT_FULL )
        return info.fullText();
      else
        return info.shortText();
    }
    else
      return "";
  }
  
  /**
  * Returns the error status. Valid values are:-
  * <ul>
  *   <li>ERROR_STATUS_NONE
  *   <li>ERROR_STATUS_MAIN
  *   <li>ERROR_STATUS_BOTH
  * </ul>
  */
  public int status()
  {
    return m_status;
  }

  // Methods for use within package

  /**
  * Sets the error status.
  *
  * @param status   new error status.
  */
  void setStatus( int status )
  {
    m_status = status;
  }
  
  /**
  * Returns true if primary error information is set.
  */
  boolean hasPrimaryInfo()
  {
    return (m_status & Roma.ERROR_STATUS_PRIMARY) != 0;
  }
  
  /**
  * Returns the primary error information.
  */
  RomaErrorInformation primaryInfo()
  {
    return m_primInfo;
  }
  
  /**
  * Returns true if secondary error information has been set.
  * (i.e. status() == STATUS_BOTH)
  */
  boolean hasSecondaryInfo()
  {
    return m_status == Roma.ERROR_STATUS_BOTH;
  }
  
  /**
  * Returns the secondary error information.
  */
  RomaErrorInformation secondaryInfo()
  {
    return m_secInfo;
  }
  
  /**
  * Receives a KxcError object and sets it up using the state
  * of this object.
  *
  * @param err  KxcError instance - ie, an encapsulation of a
  *             KXCERROR structure in the C level layer.
  * 
  * @see #RomaException( KxcError )
  * 
  */
  void setPeer( KxcError err )
  {
    err.setStatus( m_status );
    
    if ((m_status & Kxc.ERRSTAT_MAIN) != 0)
      m_primInfo.setPeer( err.result() );
    
    if ((m_status & Kxc.ERRSTAT_SUB) != 0)
      m_secInfo.setPeer( err.subResult() );
  }
  
  private static String createErrorText( KxcError err )
  {
    return
      "Roma error status: "+err.status()+
      ", main info: "+subErrorText( err.result() )+
      ", sub info: "+subErrorText( err.subResult() )
      ;
  }
  
  private static String subErrorText( KxcSubError res )
  {
    return res == null || res.fullText() == null? "": res.fullText();
  }

  /**
  * @serial
  */
  int m_status;
  
  /**
  * @serial
  */
  RomaErrorInformation m_primInfo = new RomaErrorInformation();
  
  /**
  * @serial
  */
  RomaErrorInformation m_secInfo = new RomaErrorInformation();
  
}

//--------------------------------------------------------------------
// End of file. RomaException.java.
//--------------------------------------------------------------------
