//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaExceptionMessage.java
//
// Defines      : class RomaExceptionMessage
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.util.*;
import java.io.*;

/**
* This class is used to report erroneous messages to an
* Exception Messgage Queue by encapsulating the message that
* resulted in the error plus information about the error.
*/
public class RomaExceptionMessage
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";

  private int     m_elements    = 0;
  private Date    m_exTimeStamp = null;
  private byte[]  m_userData    = null;

  private RomaExceptionError  m_error = new RomaExceptionError();

  // RomaExceptionMessage has a RomaMessage
  // Package visibility so other classes can access
  RomaMessage m_Message = new RomaMessage();

  /**
  * Constructs a new exception message with no information.
  */
  public RomaExceptionMessage()
  {
  }

  /**
  * Constructs a new message exception object using an existing
  * RomaClientRequest.
  *
  * @param message a RomaClientRequest object.
  */
  public RomaExceptionMessage( RomaClientRequest message )
  {
    m_Message = message.m_Message;
  }
  
  /**
  * Constructs a new message exception object using an existing
  * RomaClientResponse.
  *
  * @param message a RomaClientResponse object.
  */
  public RomaExceptionMessage( RomaClientResponse message )
  {
    m_Message = message.m_Message;
  }
  
  /**
  * Constructs a new message exception object using an existing
  * RomaBeRequest.
  *
  * @param message a RomaBeRequest object.
  */
  public RomaExceptionMessage( RomaBeRequest message )
  {
    m_Message = message.m_Message;
  }

  /**
  * Constructs a new message exception object using an existing
  * RomaBeResponse.
  *
  * @param message a RomaBeResponse object.
  */
  public RomaExceptionMessage( RomaBeResponse message )
  {
    m_Message = message.m_Message;
  }
  
  /**
  * Constructs a new message exception object using an existing
  * RomaStorable.
  *
  * @param message a RomaStorable object.
  */
  public RomaExceptionMessage( RomaStorable message )
  {
    m_Message = message.m_Message;
  }

  /**
  * Constructs a new message exception object using an existing
  * RomaPublication.
  *
  * @param message a RomaPublication object.
  */
  public RomaExceptionMessage( RomaPublication message )
  {
    m_Message = message.m_CR.m_Message;
  }
  
  /**
  * Constructs a new message exception object with supplied user data.
  *
  * @param buffer contains message data
  * @param bufLen length of message data
  */
  public RomaExceptionMessage( byte[] buffer,
                               int    bufLen )
  {
    setUserBuffer( buffer,  bufLen );
  }

  /**
  * Constructs a new message exception object with supplied user data.
  *
  * @param buffer contains message data
  */
  public RomaExceptionMessage(byte[] buffer)
  {
    setUserBuffer(buffer);
  }

  //== RomaMesssage Access functions ==================================

  /**
  * Set the message priority.
  * @param priority  this can be one of the following :-
  * <ul>
  * <li>PRIORITY_DEFAULT - use the default priority
  *     from the transport.
  * <li>PRIORITY_PROFILE - use the priority as specified in the
  *     associated Put API profile. If no such profile or attribute,
  *     use PRIORITY_DEFAULT.
  * <li>a specific integer meaningful to the underlying transport
  *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ
  * </ul>
  */
  public void setPriority( int priority )
  {
    m_Message.setPriority(priority);
  }
  
  /**
  * Set a new value for the message perisitence.
  * @param  persistence  this can be one of the following :-
  *   <ul>
  *   <li>PERSISTENCE_YES
  *   <li>PERSISTENCE_NO
  *   <li>PERSISTENCE_PROFILE - use the persistence value as
  *       specified in the associated Put API profile. If no such
  *       profile or attribute, use PERSISTENCE_NO.
  *   </ul>
  */
  public void setPersistence( int persistence )
  {
    m_Message.setPersistence(persistence);
  }
  
  /**
  * Set a new message format value.
  * @param format  identifies the format of message. If the 
  *   value FORMAT_PROFILE is used then the message format value
  *   is retrieved from the associated Put API profile. If the
  *   profile is not available or else has no format attribute 
  *   value then the message format value FORMAT_BINARY is used.
  */
  public void setFormat( int format )
  {
    m_Message.setFormat(format);
  }
  
  /**
  * Sets a new message feedback code.
  * @param feedback  new message feedback value.
  */
  public void setFeedback( int feedback )
  {
    m_Message.setFeedback(feedback);
  }
  
  /**
  * Set a new message expiry time.
  * @param expiry  the new expiry time value
  *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
  *   If EXPIRY_PROFILE is specified then
  *   the expiry value is retrieved from the associated Put API
  *   profile. If the profile is not available or else has no expiry
  *   attribute set, the EXPIRY_NEVER constant is used.
  */
  public void setExpiry( int expiry )
  {
    m_Message.setExpiry(expiry);
  }
  
  /**
  * Set a new message codepage value. 
  * @param codepage  the new codepage value. If CODEPAGE_PROFILE is
  *   specified then the codepage will be retrieved from the 
  *   associated Put API profile. If the profile is not available or
  *   else has no codepage attribute set, the default value of 
  *   CODEPAGE_USER will be used.
  */
  public void setCodepage( int codepage )
  {
    m_Message.setCodepage(codepage);
  }
  
  /**
  * Set the message encoding.
  * @param  encoding  the new encoding indicator. If this is 
  *   ENCODING_PROFILE then the value is retrieved from the 
  *   associated Put API profile. If the profile is not available
  *   or else has no encoding attribute set, the default value
  *   ENCODING_DEFAULT is used.
  */
  public void setEncoding( int encoding )
  {
    m_Message.setEncoding(encoding);
  }
  
  /**
  * Set the message identifier.
  * @param  msgId  a byte array containing the new message id value.
  * The msg id must not exceed kxc.MSG_ID_LENGTH.
  */
  public void setMsgId( byte[] msgId )
  {
    m_Message.setMsgId(msgId);
  }
  
  /**
  * Return the message identifier in a byte array.
  */
  public byte[] msgId()
  {
    return m_Message.msgId();
  }
  
  /**
  * Set the message correlation identifier.
  * @param  correlId  the new correlation id.
  */
  public void setCorrelId( byte[] correlId )
  {
    m_Message.setCorrelId(correlId);
  }
  
  /**
  * Return the message correlation identifier.
  */
  public byte[] correlId()
  {
    return m_Message.correlId();
  }
  
  /**
  * Sets the format name.
  *
  * @param formatName the new format name.
  */
  public void setFormatName( String formatName )
  {
    m_Message.setFormatName( formatName );
  }
  
  /**
  * Set the name of this message
  * @param messageName name to be used
  */
  public void setMsgName( String messageName )
  {
    m_Message.setMessageName(messageName);
  }

  /**
  * Set buffer containing the message data and the length of the
  * message for a PUT request. To send an empty message set length
  * to 0 and buffer to null
  *
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message.
  *
  * @return true if successfull,
  *         false if datalength < 0 or datalength = 0 and buffer != null
  */
  public boolean setMsgBuffer( byte buffer[],
                               int  dataLength )
  {
    return m_Message.setMsgBuffer( buffer, dataLength );
  }

  /**
  * Set buffer containing the message data for a PUT request.
  * To send an empty message set buffer to null.
  *
  * @param buffer     byte array containing message data.
  *
  * @return true if successfull.
  */
  public boolean setMsgBuffer(byte[] buffer)
  {
    return m_Message.setMsgBuffer(buffer, buffer.length);
  }

  /**
  * Set buffer containing the message data for a PUT request.
  *
  * @param buffer   String containing message data.
  *
  * @return true if successfull.
  */
  public boolean setMsgBuffer(String buffer)
  {
    return setMsgBuffer(buffer.getBytes());
  }

  /**
  * Set buffer containing the tag data and the length of the
  * tag for a PUT request.
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message.
  *
  * @return true if successfull,
  *         false if datalength < 0 or datalength = 0 and buffer != null
  */
  public boolean setTagBuffer( byte buffer[],
                               int  dataLength )
  {
    return m_Message.setTagBuffer( buffer, dataLength );
  }

  /**
  * Set buffer containing the tag data for a PUT request.
  *
  * @param buffer     byte array containing tag data.
  *
  * @return true if successfull.
  */
  public boolean setTagBuffer(byte[] buffer)
  {
    return m_Message.setTagBuffer(buffer, buffer.length);
  }

  /**
  * Set buffer containing the tag data for a PUT request.
  *
  * @param buffer   String containing tag data.
  *
  * @return true if successfull.
  */
  public boolean setTagBuffer(String buffer)
  {
    return setTagBuffer(buffer.getBytes());
  }

  // End of RomaMessage setting methods

  /**
  * Sets which element should be present in the exception message.
  * A combination of the following flags can be used :-
  * <ul>
  *   <li>ELEMENT_TIMESTAMP - Roma provides timestamp
  *   <li>ELEMENT_CONFIG - Roma provides own configuration
  *   <li>ELEMENT_SYSCONFIG - Roma provides system configuration
  *   <li>ELEMENT_ERROR - error information is provided
  *   <li>ELEMENT_USERDATA - user data is provided
  *   <li>ELEMENT_MESSAGE - the base message contains meaningful
  *         information
  * </ul>
  * <p>Or else one of the following constant values :-
  * <ul>
  *   <li>ELEMENT_NONE - no information to be provided.
  *   <li>ELEMENT_ALL_PUT - all information will be provided.
  * </ul>
  */
  public void setElements( int elements )
  {
    m_elements = elements;
  }
  
  /**
  * set call type
  *
  * @param type  integer value representing the type of API
  *              call being made when the error occurred.
  */
  public void setCallType( int type )
  {
    m_error.setCallType( type );
  }
  
  /**
  * Sets the error status.
  *
  * @param status   new error status.
  */
  public void setStatus( int status )
  {
    m_error.error().setStatus( status );
  }

  /**
  * set completion code
  *
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *
  * @param code completion code	
  *
  */
  public void setCompCode( int errorSubclass, 
                           int code )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setCompCode( code );
    else
      m_error.error().m_secInfo.setCompCode( code );
  }
  
  /**
  * set return code
  *
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *
  * @param code return code	
  */
  public void setReturnCode( int errorSubclass, 
                             int code )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setReturnCode( code );
    else
      m_error.error().m_secInfo.setReturnCode( code );
  }

  /**
  * Sets the reason code.
  *
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *
  * @param reason  the new reason code value.
  */
  public void setReason( int errorSubclass, 
                         int reason )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setReason( reason );
    else
      m_error.error().m_secInfo.setReason( reason );
  }

  /**
  * Sets the operation type value.
  *	
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *
  * @param opType  the new operation type identifier.
  */
  public void setOperationType( int errorSubclass, 
                                int opType )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setOperationType( opType );
    else
      m_error.error().m_secInfo.setOperationType( opType );
  }
  
  /**
  * Sets error position.
  *	
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *
  * @param position the error position
  */
  public void setPosition( int errorSubclass, 
                           int position )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setPosition( position );
    else
      m_error.error().m_secInfo.setPosition( position );
  }

  /**
  * Sets error type.
  *
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *	
  * @param type the error type
  */
  public void setErrorType( int errorSubclass, 
                            int type )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setErrorType( type );
    else
      m_error.error().m_secInfo.setErrorType( type );
  }

  /**
  * Sets additional information.
  *
  * @param errorSubclass SUBCLASS_PRIMARY   - set primary info.
  *	                     SUBCLASS_SECONDARY - set secondary info.
  *
  * @param location additional error location information.
  */
  public void setAdditionalInfo( int    errorSubclass, 
                                 String location )
  {
    if ( errorSubclass == Roma.SUBCLASS_PRIMARY )
      m_error.error().m_primInfo.setAdditionalInfo( location );
    else
      m_error.error().m_secInfo.setAdditionalInfo( location );
  }

  /**
  * Sets user data buffer.
  *
  * @param buffer user data buffer.
  * @param length user data buffer length
  * @return true if parameters are correct
  */
  public boolean setUserBuffer( byte[]  buffer,
                                int     length )
  {
    // The length parameter is included for consistancy
    // with the C++ model, for java the length is
    // an attribute of the byte array buffer.
    if ( (buffer == null && length != 0) ||
         (buffer != null && buffer.length < length) )
      return false;
    else
    {
      m_userData = buffer;
      return true;
    }
  }

  /**
  * Sets user data buffer.
  *
  * @param buffer user data buffer.
  * @return true if parameter is correct.
  */
  public boolean setUserBuffer(byte[] buffer)
  {
      m_userData = buffer;
      return true;
  }

  /**
  * Sets user data.
  *
  * @param data user data.
  * @return true if parameter is correct.
  */
  public boolean setUserBuffer(String data)
  {
      return setUserBuffer(data.getBytes());
  }

  /**
  * Timestamp when the exception message was put.
  */
  Date exceptionTimestamp()
  {
    return m_exTimeStamp;
  }

  /**
  * Returns error information relating to the exception.
  */
  RomaExceptionError error()
  {
    return m_error;
  }
  
  /**
  * Sets error information related to the exception.
  *
  * @param error RomaExceptionError
  */
  void setError( RomaExceptionError error )
  {
    m_error = error;
  }
  
  /**
  * Return the message theme. Theme can be set internally when
  * the message is being sent or received or else explicitly by
  * the user.
  */
  public RomaTheme theme()
  {
    return m_Message.theme();
  }
  
  
  
  /**
  * Non-public preparation for the put exception message operation.
  *
  * @param ctx KxcContext 
  *
  * @see RomaMessage#preparePut( KxcContext )
  */
  void preparePut( KxcContext ctx )
    throws RomaError
  {
    setPeerException( ctx.exception() );
  }
  
  /**
  * Process the put exception message operation results.
  *
  * @param ctx KxcContext
  *
  * @see RomaMessage#processPut( KxcContext )
  */
  void processPut( KxcContext ctx )
  {
    init( ctx.exception(), false );
  }
  
  /**
  * Receives a KxcPutCtrl object ( a simple wrapper around a 
  * KXCPUTOPT structure in the C API layer ) and sets its state 
  * flags according to the member data of this object.
  *
  * @param ctrl a KxcPutCtrl object.
  *
  * @see com.candle.roma.kxc.raw.KxcPutCtrl
  */
  void syncPutCtrl( KxcPutCtrl ctrl )
  {
    int elems[] = { Roma.ELEMENT_TIMESTAMP, 
                    Roma.ELEMENT_CONFIG, 
                    Roma.ELEMENT_SYSCONFIG,
                    Roma.ELEMENT_ERROR, 
                    Roma.ELEMENT_USERDATA
                  };
    
    int flags[] = { Kxc.POPT_EXCTIMESTAMP, 
                    Kxc.POPT_EXCROMADATA,
                    Kxc.POPT_EXCSYSDATA, 
                    Kxc.POPT_EXCERRDATA,
                    Kxc.POPT_EXCUSERDATA
                  };
    
    int opt = ctrl.opts();
    
    for (int i = 0; i < elems.length; i++)
    {
      if ( (m_elements & elems[i]) != 0 )
        opt |= flags[i];
    }
    
    ctrl.setOpts( opt );
  }
  
  /**
  * Issues the error report.
  *
  * @param ctx a KxcContext object.
  * @throws RomaError if operation fails.
  *
  * @see com.candle.roma.kxc.raw.KxcContext
  */
  void report( KxcContext ctx )
    throws RomaError
  {
    // always pass message to pipe, it decides whether to include or not
    m_Message.setPeer( ctx.msg(), true );
    
    ctx.reportCtrl().setThemeHandle( m_Message.theme().handle() );
    setPeerException( ctx.exception() );
    
    //----------------------------------------------------------------
    // Report the error.
    //----------------------------------------------------------------
    RomaContext.kxcOper( 
                ctx.reportError( (m_elements & Roma.ELEMENT_MESSAGE) != 0 ));
    
    init( ctx.exception(), false );
  }
  
  void init( KxcMsgInfo msgInfo )
    throws IOException
  {
    init( msgInfo.exception(), true );
    int status = msgInfo.status();
    m_Message.setTheme( (status & Kxc.MSGPART_HTHEME) != 0?
      new RomaTheme( msgInfo.themeHandle() ): null
      );
    if ((status & Kxc.MSGPART_MESSAGE) != 0)
      m_Message.initMsg( msgInfo.msg() );
    
    m_elements |= ((status &(Kxc.MSGPART_MESSAGE |
      Kxc.MSGPART_WORKFLOW)) << 8);
  }

  private void init( KxcException exc, boolean initUserData )
  {
    int el = exc.excFlags();
   	m_exTimeStamp = (el & Kxc.EXC_TIMESTAMP) != 0?
      new Date( exc.timeStamp() ): null;
    if (initUserData)
    {
      m_elements = el;
      if ( (m_elements & Kxc.EXC_ERRORDATA) != 0 )
        m_error = new RomaExceptionError( exc.errData() );
      if ( (m_elements & Kxc.EXC_USERDATA) != 0 )
        m_userData = exc.userData().data();
    }
  }
  
  /**
  * Receives a low level KxcException object and sets its 
  * state using this object's instance data.
  *
  * @param exc a KxcException object. This is just a simple wrapper
  *   around a KXCEXCPETION structure at the C API level. 
  *
  * @see com.candle.roma.kxc.raw.KxcException
  */
  private void setPeerException( KxcException exc )
  {
    //----------------------------------------------------------------
    // Mask off the low bye of the integer m_elements member and 
    // set the exception flags of the input KxcException object.
    //----------------------------------------------------------------
    exc.setExcFlags( m_elements & 0XFF );
    
    //----------------------------------------------------------------
    // Set user data.
    //----------------------------------------------------------------
    if ( m_userData != null )
      exc.setUserData( m_userData );
    
    m_error.setPeer( exc.errData() );
    
    if ( (m_elements & Roma.ELEMENT_USERDATA) != 0 && 
          m_userData == null )
      m_userData = exc.userData().data();
  }
}

//--------------------------------------------------------------------
// End of file. RomaExceptionMessage.java.
//--------------------------------------------------------------------
