//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaMessage.java
//
// Defines      : class RomaMessage
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.util.*;


/**
* This class encapsulates a message to be sent or received by Roma.
* A message consists of not only message data but also of attributes
* which describe the message such as the message identifier,
* feedback and encoding. These details help to identify the message
* data so that the appropriate unmarshalling can be carried out.
* <p>It is intended that applications will not deal with RomaMessage
* instances directly but with specializations of this class such as
* ClientRequest.
*
* @see RomaTheme
*
*/

class RomaMessage
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";
  
  private int m_type            = Roma.DATAGRAM,
              m_priority        = Roma.PRIORITY_PROFILE,
              m_persistence     = Roma.PERSISTENCE_PROFILE,
              m_format          = Roma.FORMAT_PROFILE,
              m_feedback        = 0,
              m_expiry          = Roma.EXPIRY_PROFILE,
              m_codepage        = Roma.CODEPAGE_PROFILE,
              m_encoding        = defaultEncoding(),
              m_securityStatus  = Roma.STATUS_NONE,
              m_backoutCount    = Roma.BACKOUT_UNSUPPORTED;
  
  String          m_subject;
  private String  m_formatName;
  private Date    m_timeStamp;
  private byte    m_msgId[];
  private byte    m_correlId[];
  private String  m_MsgName;
  
  private RomaSecurityDef m_sec = null;
  
  private RomaTheme       m_theme = new RomaTheme();
  
  private byte            m_msgBuffer[] = null;
  private RomaInteger     m_msgLength   = new RomaInteger(-1);
  
  private byte            m_tagBuffer[] = null;
  private RomaInteger     m_tagLength   = new RomaInteger(-1);
  
  private long m_incomingILSDataHandle = Kxc.NULL_ILS_DATA_HANDLE;
  
  /**
  * Default constructor. Sets message object up with default
  * attributes.
  */
  public RomaMessage()
  {
  }
  
  /**
  * Copy constructor.
  */
  public RomaMessage( RomaMessage msg )
  {
    m_type                  = msg.m_type;
    m_priority              = msg.m_priority;
    m_persistence           = msg.m_persistence;
    m_format                = msg.m_format;
    m_feedback              = msg.m_feedback;
    m_expiry                = msg.m_expiry;
    m_codepage              = msg.m_codepage;
    m_encoding              = msg.m_encoding;
    m_securityStatus        = msg.m_securityStatus;
    m_backoutCount          = msg.m_backoutCount;
    m_formatName            = msg.m_formatName;
    m_msgId                 = msg.m_msgId;
    m_correlId              = msg.m_correlId;
    m_theme                 = new RomaTheme( msg.m_theme.handle() );
    m_timeStamp             = msg.m_timeStamp;
    m_subject               = msg.m_subject;
    m_incomingILSDataHandle = msg.m_incomingILSDataHandle;
    
    // Shallow copy ...
    m_MsgName   = msg.m_MsgName;
    m_msgBuffer = msg.m_msgBuffer;
    m_msgLength = msg.m_msgLength;
    m_tagBuffer = msg.m_tagBuffer;
    m_tagLength = msg.m_tagLength;
  }
  
  /**
  * Returns the encoding for this operating system.
  */
  public static final int defaultEncoding()
  {
    return KxcConnection.nativeConnection().platformEncoding();
  }
  
  /**
  * Return the message theme. Theme can be set internally when
  * the message is being sent or received or else explicitly by
  * the user.
  */
  public RomaTheme theme()
  {
    return m_theme;
  }
  
  /**
  * Update the message theme.
  *
  * @param theme RomaTheme object.
  */
  public void setTheme( RomaTheme theme )
  {
    m_theme = theme;
  }
  
  /**
  * Return the message type. This can be one of the following :-
  * <ul>
  * <li>DATAGRAM
  * <li>REQUEST
  * <li>RESPONSE
  * <li>NOTIFICATION
  * </ul>
  * Note that these are values returned to indicate the type of 
  * an incoming message.  An application cannot set the message
  * type in order to create a specific type of message.  The
  * type of an outgoing message is defined implicitly by the
  * options selected when the message is sent.
  */
  public int msgType()
  {
    return m_type;
  }
  
  /**
  * Returns the priority assigned to the message.
  */
  public int priority()
  {
    return m_priority;
  }
  
  /**
  * Set the message priority.
  * @param priority  this can be one of the following :-
  * <ul>
  * <li>PRIORITY_DEFAULT - use the default priority
  *     from the transport.
  * <li>PRIORITY_PROFILE - use the priority as specified in the 
  *     associated Put API profile. If no such profile or attribute,
  *     use PRIORITY_DEFAULT.
  * <li>a specific integer meaningful to the underlying transport
  *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ 
  * </ul>
  */
  public void setPriority( int priority )
  {
    m_priority = priority;
  }
  
  /**
  * return the message perisitence value.
  */
  public int persistence()
  {
    return m_persistence;
  }
  
  /**
  * Set a new value for the message persistence.
  * @param  persistence  this can be one of the following :-
  *   <ul>
  *   <li>PERSISTENCE_YES
  *   <li>PERSISTENCE_NO
  *   <li>PERSISTENCE_PROFILE - use the persistence value as
  *       specified in the associated Put API profile. If no such
  *       profile or attribute, use PERSISTENCE_NO.
  *   </ul>
  */
  public void setPersistence( int persistence )
  {
    m_persistence = persistence;
  }
  
  /**
  * Return the message format.
  */
  public int format()
  {
    return m_format;
  }
  
  /**
  * Set a new message format value.
  * @param format  identifies the format of message. If the 
  *   value FORMAT_PROFILE is used then the message format value
  *   is retrieved from the associated Put API profile. If the
  *   profile is not available or else has no format attribute 
  *   value then the message format value FORMAT_BINARY is used.
  */
  public void setFormat( int format )
  {
    m_format = format;
  }
  
  /**
  * Get the message feedback code.
  */
  public int feedback()
  {
    return m_feedback;
  }
  
  /**
  * Sets a new message feedback code.
  * @param feedback  new message feedback value.
  */
  public void setFeedback( int feedback )
  {
    m_feedback = feedback;
  }
  
  /**
  * Return the message expiry time.
  */
  public int expiry()
  {
    return m_expiry;
  }
  
  /**
  * Set a new message expiry time.
  * @param expiry  the new expiry time value
  *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
  *   If EXPIRY_PROFILE is specified then
  *   the expiry value is retrieved from the associated Put API
  *   profile. If the profile is not available or else has no expiry
  *   attribute set, the EXPIRY_NEVER constant is used.
  */
  public void setExpiry( int expiry )
  {
    m_expiry = expiry;
  }
  
  /**
  * Return the message codepage.
  */
  public int codepage()
  {
    return m_codepage;
  }
  
  /**
  * Set a new message codepage value. 
  * @param codepage  the new codepage value. If CODEPAGE_PROFILE is
  *   specified then the codepage will be retrieved from the 
  *   associated Put API profile. If the profile is not available or
  *   else has no codepage attribute set, the default value of 
  *   CODEPAGE_USER will be used.
  */
  public void setCodepage( int codepage )
  {
    m_codepage = codepage;
  }
  
  /**
  * Get the message encoding.
  */
  public int encoding()
  {
    return m_encoding;
  }
  
  /**
  * Set the message encoding.
  * @param  encoding  the new encoding indicator. If this is
  *   ENCODING_PROFILE then the value is retrieved from the 
  *   associated Put API profile. If the profile is not available
  *   or else has no encoding attribute set, the default value
  *   ENCODING_DEFAULT is used.
  */
  public void setEncoding( int encoding )
  {
    m_encoding = encoding;
  }
  
  /**
  * Return the message identifier in a byte array.
  */
  public byte[] msgId()
  {
    return m_msgId;
  }
  
  /**
  * Set the message identifier.
  * @param  msgId  a byte array containing the new message id value.
  * The msg id must not exceed kxc.MSG_ID_LENGTH.
  */
  public void setMsgId( byte[] msgId )
  {
    m_msgId = msgId;
  }
  
  /**
  * Return the message correlation identifier.
  */
  public byte[] correlId()
  {
    return m_correlId;
  }
  
  /**
  * Set the message correlation identifier.
  * @param  correlId  the new correlation id.
  */
  public void setCorrelId( byte[] correlId )
  {
    m_correlId = correlId;
  }
  
  /**
  * Return message data buffer.
  */
  public byte[] msgBuffer()
  {
    return m_msgBuffer;
  }
  
  /**
  * Return message data buffer length.
  */
  public int msgDataLength()
  {
    return m_msgLength.intValue();
  }
  
  /**
  * Return message tag buffer.
  */
  public byte[] tagBuffer()
  {
    return m_tagBuffer;
  }
  
  /**
  * Return message data buffer length.
  */
  public int tagDataLength()
  {
    return m_tagLength.intValue();
  }
  
  /**
  * Return the message status.
  */
  public int status()
  {
    return m_securityStatus;
  }
  
  /**
  * Return the backout count. This is the number 
  * of times that the message has been rolled back.
  */
  public int backoutCount()
  {
    return m_backoutCount;
  }
  
  /**
  * Return the message format name.
  */
  public String formatName()
  {
    return m_formatName;
  }
  
  /**
  * Sets the format name.
  *
  * @param formatName the new format name.
  */
  public void setFormatName( String formatName )
  {
    m_formatName = formatName;
    if (m_formatName != null)
      m_format = Roma.FORMAT_USENAME;
  }
  
  /**
  * Returns the time stamp which, for get operations, is the 
  * time at which the put operation occurred.
  */
  public Date timeStamp()
  {
    return m_timeStamp;
  }
  
  /**
  * Set the name of this message
  * @param messageName name to be used
  */
  public void setMessageName( String messageName )
  {
    m_MsgName = messageName;
  }
  
  /**
  * Returns name of message
  */
  public String messageName( )
  {
    return m_MsgName;
  }
  
  /**
  * Returns subject name of message
  */
  public String subject( )
  {
    return m_subject;
  }
		
  /**
  * Set buffer containing the message data and the length of the 
  * message for a PUT request.
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message. 
  *
  * @return true if set successfully. 
  */
  public boolean setMsgBuffer( byte buffer[],
                               int  dataLength )
  {
    // No -ve values. Allow zero length for use as a PROD
    if ( dataLength < 0 || 
         ( dataLength > 0 && buffer == null ))
      return false;

    m_msgBuffer = buffer;
    
    // length is held as a RomaInteger so user can 
    // supply as a return parameter on GET operations 
    m_msgLength = new RomaInteger(dataLength);  
    
    return true;
  }
  
  /**
  * Set buffer containing the message data and the length of the 
  * message for a PUT request.
  * 
  * This version for use within package, accepting RomaInteger for
  * length parameter.
  *
  * @param buffer     byte array containing the message.
  * @param dataLength length of the message.
  *
  * @return true if set successfully.
  */
  boolean setMsgBuffer( byte        buffer[],
                        RomaInteger dataLength )
  {
    // No -ve values. Allow zero length for use as a PROD
    if ( dataLength.intValue() < 0 || 
         ( dataLength.intValue() > 0 && buffer == null ))
      return false;

    m_msgBuffer = buffer;
    m_msgLength = dataLength;
    
    return true;
  }
  
  /**
  * Set buffer containing the tag data and the length of the
  * tag for a PUT request.
  * @param buffer     byte array containing the tag.
  * @param dataLength length of the tag.
  *
  * @return true if set successfully.
  */
  public boolean setTagBuffer( byte buffer[],
                               int  dataLength )
  {
    // No -ve values. Allow zero length for use as a PROD
    if ( dataLength < 0 || 
         ( dataLength > 0 && buffer == null ))
      return false;
    
    m_tagBuffer = buffer;
    
    // length is held as a RomaInteger so user can
    // supply as a return parameter on GET operations
    m_tagLength = new RomaInteger(dataLength);
    
    return true;
  }
  
  /**
  * Set buffer containing the tag data and the length of the
  * tag for a PUT request.
  *
  * This version for use within package, accepting RomaInteger for
  * length parameter.
  *
  * @param buffer byte array containing the message.
  * @param dataLength length of the message. 
  * 
  * @return true if set successfully. 
  */
  boolean setTagBuffer( byte        buffer[],
                        RomaInteger dataLength )
  {
    // No -ve values. Allow zero length for use as a PROD
    if ( dataLength.intValue() < 0 || 
         ( dataLength.intValue() > 0 && dataLength == null ))
      return false;

    m_tagBuffer = buffer;
    m_tagLength = dataLength;
    
    return true;
  }
  
  /**
  * Sets the security definition. Subsequent put() and get()
  * operations will pass this to Roma inline services.
  *
  * @param sec new security definition. May be null.
  *
  * @see RomaSecurityDef
  */
  public void setSecurityDef( RomaSecurityDef sec )
  {
    m_sec = sec;
  }
  
  /**
  * Returns current security buffer associated with this context.
  */
  byte[] getSecurityBuffer()
    throws RomaError
  {
    if ( m_sec == null )
      return null;
    
    return m_sec.getBuffer();
  }
  
  /**
  * get incoming ils data handle
  */
  long getILSDataHandle()
  {
    return  m_incomingILSDataHandle;
  }
  
  /**
  * Provided to allow RomaContext to set the required
  * buffer lengths after
  *
  *
  * @param msgLength length of message buffer
  * @param tagLength length of tag buffer
  *
  */
  void setReturnedBufferLengths( int msgLength,
                                 int tagLength )
  {
    m_msgLength.setValue( msgLength );
    m_tagLength.setValue( tagLength );
  }
  
  /**
  * Prepares message for a put operation.
  * Sets up the given KxcMsg object with values of the message
  * attributes. Also sets up buffers.
  *
  * @param ctx KxcContext 
  */
  void preparePut( KxcContext ctx )
    throws RomaError
  {
    KxcPutCtrl ctrl = ctx.putCtrl();
    
    //--------------------------------------------------------------
    // Set up "peer" KxcMsg object in the "raw" layer. Passing in
    // false for the second argument means that only some of the
    // KxcMsg object's members get updated from this object.
    //--------------------------------------------------------------
    setPeer( ctx.msg(), false );
    // set message ids
    ctrl.setMsgId( m_msgId );
    ctrl.setCorrelId( m_correlId );
    ctrl.setThemeHandle( m_theme != null ? m_theme.handle(): Kxc.NULL_HANDLE );
  }
  
  /**
  * Prepares message for a get operation .
  * Set message and tag buffers and lengths from
  * parameters. A -1 value for length implies auto-allocate
  */
  void prepareGet( KxcContext ctx )
  {
    ctx.msg().msgBuffer().setDataLength( m_msgLength.intValue() );
    ctx.msg().setMsgLength( m_msgLength.intValue() ) ;
    
    if (m_msgLength.intValue() != -1)	  
      ctx.msg().msgBuffer().setData( m_msgBuffer );
    
    ctx.msg().tagBuffer().setDataLength( m_tagLength.intValue() );
    ctx.msg().setTagLength( m_tagLength.intValue() ) ;
    
    if (m_tagLength.intValue() != -1)
      ctx.msg().tagBuffer().setData( m_tagBuffer );
  }
  
  /**
  * Process message put operation results.
  *
  * @param ctx KxcContext
  */
  void processPut( KxcContext ctx )
  {
    KxcPutCtrl ctrl = ctx.putCtrl();
    KxcMsg msg      = ctx.msg();
    
    // remember attributes which are set as result of
    // the put operation
    m_type      = msg.type();
    m_expiry    = msg.expiry();
    m_msgId     = msg.msgId();
    m_correlId  = msg.correlId();
    m_timeStamp = new Date( msg.timeStamp() );
    
    // process put options and update message theme
    // accordingly
    m_theme = new RomaTheme( ctrl.themeHandle() );
  }
  
  /**
  * Process message get operation results.
  *
  * @param ctx KxcContext
  */
  void processGet( KxcContext ctx )
  {
    //----------------------------------------------------------------
    // The initMsg() call updates this object's attributes with the
    // member data of the KxcMsg object returned by the msg() call
    // on the ctx.
    //----------------------------------------------------------------
    initMsg( ctx.msg() );
    // process raw attributes
    // process options
    m_theme = new RomaTheme( ctx.getCtrl().themeHandle() );
    
    // Keep incoming ils data handle for use by BeRequest and ClientResp 
    KxcPutCtrl ctrl = ctx.putCtrl();
    m_incomingILSDataHandle =  ctrl.ilsDataHandle();
  }
  
  /**
  * Updates the internal state of this RomaMessage object from 
  * the state of a KxcMsg object.
  *
  * @param msg a KxcMsg object.
  *
  * @see com.candle.roma.kxc.raw.KxcMsg
  */
  void initMsg( KxcMsg msg )
  {
    m_type            = msg.type();
    m_priority        = msg.priority();
    m_persistence     = msg.persistence();
    m_format          = msg.format();
    m_feedback        = msg.feedback();
    m_expiry          = msg.expiry();
    m_codepage        = msg.codepage();
    if ( m_codepage <= 0 )
    {
      m_codepage      = Kxc.USEDEFAULT;
    }
    m_encoding        = msg.encoding();
    m_msgId           = msg.msgId();
    m_correlId        = msg.correlId();
    m_securityStatus  = msg.status();
    m_backoutCount    = msg.backoutCount();
    m_timeStamp       = new Date( msg.timeStamp() );
    m_MsgName         = msg.messageName();
    m_subject         = msg.subject(); 
    m_formatName      =	msg.formatName( );
    
    // return msg and tag data lengths
    m_msgLength.setValue( msg.msgLength() );	
    m_tagLength.setValue( msg.tagLength() );	
    m_msgBuffer = msg.msgBuffer().data();
    m_tagBuffer = msg.tagBuffer().data();
  }
  
  /**
  * Receives a KxcMsg object and sets it up using the state
  * of this object.
  *
  * @param msg  KxcMsg instance - ie, an encapsulation
  *   of a KXCMSG structure in the C level layer.
  * @param fullCopy  boolean indication of whether all attributes
  *   of this object are to be updated. If false then some 
  *   attributes are left untouched.
  * 
  * @see com.candle.roma.kxc.raw.KxcMsg
  */
  void setPeer( KxcMsg msg, boolean fullCopy )
  {
    // set raw properties
    msg.setPriority   ( m_priority );
    msg.setPersistence( m_persistence );
    msg.setFormat     ( m_format );
    msg.setFeedback   ( m_feedback );
    msg.setExpiry     ( m_expiry );
    msg.setCodepage   ( m_codepage );
    msg.setEncoding   ( m_encoding );
    msg.setFormatName ( m_formatName );
    msg.setMessageName( m_MsgName );
    
    /*
    * When a RomaMessage is first constructed the tag and tag lengths
    * are initialised to -1 to indicate that buffers should
    * be auto-allocated in Roma for a get request.
    *
    * For a put we need to translate -1 to 0 to indicate
    * no data (i.e. a message prod)
    */
    
    // setup message data buffer from values set by SetMsgBuffer
    msg.msgBuffer().setData( m_msgBuffer );
    int len = m_msgLength.intValue();
    
    msg.msgBuffer().setDataLength( (len == -1) ? 0 : len );
    msg.setMsgLength( (len == -1) ? 0 : len );
    
    // setup tag data buffer from values set by SetTagBuffer
    msg.tagBuffer().setData( m_tagBuffer );
    len = m_tagLength.intValue();
    msg.tagBuffer().setDataLength( (len == -1) ? 0 : len );
    msg.setTagLength( (len == -1) ? 0 : len );
    
    if (fullCopy)
    {
      msg.setType         ( m_type );
      msg.setMsgId        ( m_msgId );
      msg.setCorrelId     ( m_correlId );
      msg.setStatus       ( m_securityStatus );
      msg.setBackoutCount ( m_backoutCount );
      msg.setTimeStamp    ( m_timeStamp != null ?
                            m_timeStamp.getTime(): 0 );
    }
  }

  RomaMessage(int type, int prio, int pers, int format, int fback,
              int expiry, int cp, int enc, byte[] mid, byte[] cid,
              int status, byte[] mbuf, byte[] tbuf, int backout,
              long sec, int msec,
              String fname, String sname, String mname, long theme)
  {
    m_type                  = type;
    m_priority              = prio;
    m_persistence           = pers;
    m_format                = format;
    m_feedback              = fback;
    m_expiry                = expiry;
    m_codepage              = cp;
    m_encoding              = enc;
    m_securityStatus        = status;
    m_backoutCount          = backout;
    m_formatName            = fname;
    m_msgId                 = mid;
    m_correlId              = cid;
    m_timeStamp             = new Date(sec * 1000 + msec);
    m_subject               = sname;
    m_MsgName               = mname;
    m_msgBuffer             = mbuf;
    m_tagBuffer             = tbuf;

    if (theme != 0)
    {
        m_theme                 = new RomaTheme(theme);
    }
  }
}

