//--------------------------------------------------------------------
//
// Copyright !Candle Corporation 2001-2003
//
//--------------------------------------------------------------------
//
// Module       : RomaPublication.java
//
// Defines      : class RomaPublication
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
 * This class encapsulates publication messages sent by a Roma client.
 * Almost all of the methods provided are provided the same as the
 * RomaClientRequest class.  
 * The class also provides the setRetain method - selecting this option
 * ensures that the message is not deleted by the MQSeries broker on
 * receipt.
 */

public class RomaPublication
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 2001";

  // RomaPublication has a RomaClientRequest 
  // Package visibility so other classes can access
  RomaClientRequest m_CR        = new RomaClientRequest("");
  RomaSubject       m_Sub       = new RomaSubject ("");
  boolean           m_Retain    = false;

  /**
   * Constructs a publication with default options and with the name
   * of a Roma business service and subject.
   *
   * @param svcName  Name of a Roma Business Service.
   * @param subject  Subject of the publication.
   */
  public RomaPublication (String svcName, RomaSubject subject)
  {
    m_CR.setServiceName(svcName);
    setSubject(subject);
  }

  /**
   * Returns the name of the business service this publication is 
   * intended for. Uses package visibility - not public
  */
  String serviceName()
  {
    return m_CR.serviceName();
  }

  /**
   * Sets the name of the Roma Business Service that a publish 
   * operation will send this publication or delete publication 
   * to.
   *
   * @param svcName  Roma Business Service name.
  */
  public void setServiceName (String svcName)
  {
    m_CR.setServiceName (svcName);
  }

  /**
   * Invoked from the RomaClient.publish() method immediately prior to 
   * attempting to issue the publication to the Roma network.
   * Is intended that this method be overridden in derived classes
   * to provide an implementation for marshalling application
   * specific data into the associated message buffer.
   * <p> If this method fails the publish operation will not be carried 
   * out.
   *
   * @see RomaClient#publish(RomaPublication)
   * @exception IOException is thrown on error.
  */
  public void marshalRequest() throws IOException
  {
  }

  /**
   * Sets the put selector. This specifies which identifiers
   * contained in the message should be used when the 
   * message is published.
   * @param sel the new value of the put selector.
   * <p>
   * The value will be one of the following fields :-
   * <ul>
   *   <li>USE_NONE
   *   <li>USE_MSGID
   *   <li>USE_CORRELID
   *   <li>USE_MSGID_AND_CORRELID
   * </ul>
  */
  public void setSelector (int sel)
  { 
    m_CR.setSelector (sel); 
  }

  /**
   * Controls whether an MSMQ message can be placed outside syncpoint
   * control. If the application attempts to put a message to MSMQ inside  
   * syncpoint control but the target queue does not support transactions,
   * the call is normally rejected with error code RC_NONXACTIONAL.
   * The application can call this method with true flag to force an MSMQ
   * message to be placed outside syncpoint control.
   * @param flag  one of :-
   * <ul>
   *  <li>true - allow MSMQ messages to be placed outside syncpoint control
   *  <li>false - don't allow MSMQ messages to be placed outside syncpoint
   *              control
   * </ul>
   */
  public void setNoSyncAllowed (boolean flag)
  { 
    m_CR.setNoSyncAllowed (flag); 
  }
 
  /**
   * Associates an API profile with the publish or delete
   * publication operation.
   *
   * @param profile  the name of the API profile.
  */
  public void setApiProfileName (String profile)
  { 
    m_CR.setApiProfileName (profile); 
  }

  /**
   * Associates an error profile with the publish or delete
   * publication operation.
   *
   * @param profile  the name of the error profile.
  */
  public void setErrorProfileName (String profile)
  { 
    m_CR.setErrorProfileName (profile); 
  }

  /**
   * Specify if error profiling is to be suppressed
   *   
   * @param b true to suppress error profiling
   */
   public void setSuppressErrorProfile (boolean b)
   {
     m_CR.setSuppressErrorProfile(b);
   } 

  /**
   * Enables or disables the option to send the message to an
   * exception message service if the publish or delete 
   * publication operation fails.
   *
   * @param select  one of :-
   * <ul>
   *    <li> true - <strong>do not</strong> perform a send to an
   *                exception message service if the publish or 
   *                delete publication fails.
   *    <li> false - allow a message to be sent to an exception
   *                 message service if the operation fails.
   * </ul>
  */
  public void setNoEms (boolean select)
  { 
    m_CR.setNoEms (select); 
  }

  /**
   * Enables or disables the option to send a notification message
   * back to the sender if the message operation fails.
   *
   * @param select  one of :-
   * <ul>
   *    <li> true  - <strong>do not</strong> allow a notification
   *                 message to be sent on a get failure.
   *    <li> false - <strong>do </strong> allow a notification
   *                 message to be sent.
   * </ul>
  */
  public void setNoNotification (boolean select)
  { 
    m_CR.setNoNotification (select); 
  }

  /**
   * Enables or disables the option to send the message to a 
   * message transformation service if the next component in the
   * workflow cannot accept the message format.
   *
   * @param select  one of :-
   * <ul>
   *    <li> true - <strong>do not</strong> allow the message to
   *                sent to the transformation service.
   *    <li> false - allow the message to be routed via a
   *                 transformation service if necessary.
   * </ul>
  */
  public void    setNoTransformation (boolean select)
  { 
    m_CR.setNoTransformation (select); 
  }

  /**
   * Specify if termination is supported
   *   
   * @param b true to indicate termination is supported.
   */
  public void setTermSupported (boolean b)
  { 
    m_CR.setTermSupported (b); 
  }

  /**
   * Specify whether a new theme should be allocated.
   * <p>
   * @param b  if true then a new theme is allocated.
  */
  public void setNewTheme(boolean b) 
  { 
    m_CR.setNewTheme(b); 
  }

  /**
   * Specify whether the theme should be retained for future operations.
   * <p>
   * @param b  if true then the theme is retained.
  */
  public void setRetainTheme(boolean b)
  { 
    m_CR.setRetainTheme(b); 
  }

  /**
   * Specify whether the theme should be closed automatically when the
   * operation completes.
   * <p>
   * @param b  if true then the theme will be closed.
  */
  public void setCloseTheme(boolean b)
  { 
    m_CR.setCloseTheme(b); 
  }

  /**
   * Specify whether an existing theme should be used.
   * <p>
   * @param b  if true then the existing theme is used.
  */
  public void setUseTheme(boolean b)
  { 
    m_CR.setUseTheme(b); 
  }

  /**
   *  set or clear confirmation of delivery flag 
   */
  public void setCOD (boolean flag)
  {
    m_CR.setCOD (flag);
  }

  /**
   *  set or clear confirmation of arrival flag 
   */
  public void setCOA (boolean flag)
  {
    m_CR.setCOA (flag);
  }
  
  /**
   * Sets the datagram flag.
   * Controls type of the request sent to server.
   * <p>
   * If an API profile is to be used  (selected by invoking the
   * useApiProfile() method and verifiable by querying the
   * isApiProfileUsed() method) then this setting will be
   * overridden either by the profile or, if not available, the
   * default for this option.
   *
   * @param b  one of :-
   * <ul>
   *   <li>true - send DATAGRAM, which means that the application
   *       will receive no confirmations.
   *   <li>false - send REQUEST, which means that the application
   *       will receive confirmation that the broker has received
   *       and processed the message.
   * </ul>
  */
  public void setDatagram(boolean b)
  { 
    m_CR.setDatagram(b); 
  }
  
  /**
   * Set the message priority.
   * @param priority  this can be one of the following :-
   * <ul>
   * <li>PRIORITY_DEFAULT - use the default priority
   *     from the transport.
   * <li>PRIORITY_PROFILE - use the priority as specified in the 
   *     associated Put API profile. If no such profile or attribute,
   *     use PRIORITY_DEFAULT.
   * <li>a specific integer meaningful to the underlying transport
   *     e.g. 0..9 for MQSeries and BEAMQ,  0..7 for MSMQ 
   * </ul>
   */
  public void setPriority (int priority)
  {
    m_CR.setPriority(priority);
  }

  /**
   * Set a new value for the message perisitence.
   * @param  persistence  this can be one of the following :-
   *   <ul>
   *   <li>PERSISTENCE_YES
   *   <li>PERSISTENCE_NO
   *   <li>PERSISTENCE_PROFILE - use the persistence value as
   *       specified in the associated Put API profile. If no such
   *       profile or attribute, use PERSISTENCE_NO.
   *   </ul>
   */
  public void setPersistence (int persistence)
  {
    m_CR.setPersistence(persistence);
  }

  /**
   * Set a new message format value.
   * @param format  identifies the format of message. If the 
   *   value FORMAT_PROFILE is used then the message format value
   *   is retrieved from the associated Put API profile. If the
   *   profile is not available or else has no format attribute 
   *   value then the message format value FORMAT_BINARY is used.
   */
  public void setFormat (int format)
  {
    m_CR.setFormat(format);
  }

  /**
   * Sets a new message feedback code.
   * @param feedback  new message feedback value.
   */
  public void setFeedback (int feedback)
  {
    m_CR.setFeedback(feedback);
  }

  /**
   * Set a new message expiry time.
   * @param expiry  the new expiry time value
   *   <strong>in seconds</strong> or EXPIRY_NEVER or EXPIRY_PROFILE.
   *   If EXPIRY_PROFILE is specified then
   *   the expiry value is retrieved from the associated Put API
   *   profile. If the profile is not available or else has no expiry
   *   attribute set, the EXPIRY_NEVER constant is used.
   */
  public void setExpiry (int expiry)
  {
    m_CR.setExpiry(expiry);
  }

  /**
   * Set a new message codepage value. 
   * @param codepage  the new codepage value. If CODEPAGE_PROFILE is
   *   specified then the codepage will be retrieved from the 
   *   associated Put API profile. If the profile is not available or
   *   else has no codepage attribute set, the default value of 
   *   CODEPAGE_USER will be used.
   */
  public void setCodepage (int codepage)
  {
    m_CR.setCodepage(codepage);
  }

  /**
   * Set the message encoding.
   * @param  encoding  the new encoding indicator. If this is 
   *   ENCODING_PROFILE then the value is retrieved from the 
   *   associated Put API profile. If the profile is not available
   *   or else has no encoding attribute set, the default value
   *   ENCODING_DEFAULT is used.
   */
  public void setEncoding (int encoding)
  {
    m_CR.setEncoding(encoding);
  }

  /**
   * Set the message identifier.
   * @param  msgId  a byte array containing the new message id value.
   * The msg id must not exceed kxc.MSG_ID_LENGTH.
   */
  public void setMsgId (byte[] msgId)
  {
    m_CR.setMsgId(msgId);
  }

  /**
   * Return the message identifier in a byte array.
   */
  public byte[] msgId()
  {
    return m_CR.msgId();
  }

  /**
   * Set the message correlation identifier.
   * @param  correlId  the new correlation id.
   */
  public void setCorrelId (byte[] correlId)
  {
    m_CR.setCorrelId(correlId);
  }

  /**
   * Return the message correlation identifier.
   */
  public byte[] correlId()
  {
    return m_CR.correlId();
  }

  /**
   * Sets the format name.
   *
   * @param formatName the new format name.
   */
  public void setFormatName (String formatName)
  {
    m_CR.setFormatName (formatName);
  }

  /**
   * Update the message theme.
   *
   * @param theme RomaTheme object.
   */
  public void setTheme (RomaTheme theme)
  {
    m_CR.setTheme (theme);
  }

  /**
   * Return the message theme. Theme can be set internally when
   * the message is being sent or received or else explicitly by
   * the user.
   */
  public RomaTheme theme()
  {
    return m_CR.theme();
  }

  /**
   * Set the name of this message
   * @param messageName name to be used
   */
  public void setMsgName (String messageName)
  {
    m_CR.setMsgName(messageName);
  }

  /**
   * Set buffer containing the message data for a publish operation.
   * To send an empty message set buffer to null.
   *
   * @param buffer     byte array containing message data.
   *
   * @return true if successful
   */
  public boolean setMsgBuffer (byte[] buffer)
  {
    return m_CR.setMsgBuffer(buffer, buffer.length);
  }

  /**
   * Set buffer containing the message data and the length of the
   * message for a publish operation. To send an empty message set length
   * to 0 and buffer to null.
   *
   * @param buffer     byte array containing message data.
   * @param dataLength length of the message.
   *
   * @return true if successful,
   *         false if datalength < 0 or datalength = 0 and buffer != null
   */
  public boolean setMsgBuffer (byte   buffer[],
                               int    dataLength)
  {
    return m_CR.setMsgBuffer (buffer, dataLength);
  }

  /**
   * Set buffer containing the message data for a publish operation.
   *
   * @param buffer  String containing message data.
   *
   * @return true if successful
   */
  public boolean setMsgBuffer(String buffer)
  {
    return setMsgBuffer(buffer.getBytes());
  }

  /**
   * Set security definition
   *
   * @param secDef security definition to be sent with message
   *
   */
  public void setSecurityDef (RomaSecurityDef secDef)
  {
    m_CR.setSecurityDef (secDef);  
  }
  
  /**
   * Specify sync point definitiion
   * <p>
   * @param  sync definition to be used.
  */
  public void setSyncpointDef (RomaSyncpointDef sync)
  {
    m_CR.setSyncpointDef (sync);
  }
  
  /**
   * Set outgoing ILS data
   *
   * @param  ilsData The ILS data to be sent with message 
   */
  public void setILSData(RomaILSData ilsData)
  { 
    m_CR.setILSData(ilsData);
  }

  /**
   * Get incoming ILS data
   *
   */
  public RomaILSData getILSData()
  {
    return m_CR.getILSData();
  }

  /**
   * Set a subject of the publication.
   *
   * @param subject subject of the publication.
   */
  public void setSubject(RomaSubject subject)
  {
    m_Sub.setSubject(subject.subject());
    m_Sub.setSubjectStyle(subject.style());
  }

  /**
   * Sets the retain publication flag, which if set on MQSeries,
   * causes the MQSeries broker to retain the publication so that
   * subscribers can recieve "old" publications.
   *
   * @param b  one of :-
   * <ul>
   *   <li>true - the publication is to be retained
   *   <li>false - the publication is not to be retained
   * </ul>
  */
  public void setRetain(boolean b)
  {
    m_Retain = b;
  }
}

//--------------------------------------------------------------------
// End of file.
//--------------------------------------------------------------------

