//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcobe.hpp,v 1.17 2004/01/22 17:53:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcobe.hpp,v $
// $Revision: 1.17 $
//--------------------------------------------------------------------
// The RomaBusinessElement public interface.
//--------------------------------------------------------------------

#ifndef __KXCOBE_HPP
#define __KXCOBE_HPP

#include <kxcotb.hpp>
#include <kxcotrb.hpp>
#include <kxcoils.hpp>
#include <kxcosec.hpp>
#include <kxcogto.hpp>
#include <kxcobrs.hpp>
#include <kxcobrq.hpp>
#include <kxcoem.hpp>
#include <kxcoli.hpp>
#include <kxcoerr.hpp>
#include <kxcodac.hpp>
#include <kxcoems.hpp>

//------------------------------------------------------------------------
// The internal implementation class.
//------------------------------------------------------------------------
class RomaImplElement;


//------------------------------------------------------------------------
// The RomaBusinessElement class.
//
// Represents the application's server-side use of Roma.
//
// A RomaBusinessElement can
//      * receive requests from a Client and issue responses
//      * commit and rollback local transactions
//      * work with the Roma EMS, write logs etc
//      * work with Roma directory data
//
// The RomaBusinessElement class can be sub-classed by the application
// to differentiate different request types. The createRequest method is
// specialised to determine which type of RomaBeRequest has been received.
//------------------------------------------------------------------------
class KXCODLL RomaBusinessElement : public RomaTypeBase, 
                                    public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default Constructor.
    // It does not initialize connection. Use connect() later.
    //----------------------------------------------------------------
    RomaBusinessElement();

    //----------------------------------------------------------------
    // Construct and initialize connection.
    //----------------------------------------------------------------
    RomaBusinessElement(
        const char* name,
        KXCULONG options = Roma::APP_NONE,
        const RomaSecurityDef& secDef = RomaSecurityDef(),
        const char* modelQueueName = NULL,
        const RomaILSData& ilsData = RomaILSData(),
        const RomaDirectoryAccess& auth = RomaDirectoryAccess());

    //----------------------------------------------------------------
    // Destructor.
    // Terminates connection.
    //----------------------------------------------------------------
    virtual ~RomaBusinessElement();

    //----------------------------------------------------------------
    // Initializes Roma connection.
    //
    // Parameters:
    //   (in) name - client name
    //   (in) options - options, it is APP_NONE or any combination
    //        of the following flags:
    //        APP_EXCLUSIVE
    //        APP_MQSERIES_XACTION
    //        APP_NON_XACTIONAL
    //        APP_MQSERIES_FASTPATH
    //   (in) secDef - security definition
    //   (in) ILSData - ILS data to hand off to this application
    //----------------------------------------------------------------
    bool connect(const char* name,
                 KXCULONG options = Roma::APP_NONE,
                 const RomaSecurityDef& secDef = RomaSecurityDef(),
                 const char* modelQueueName = NULL,
                 const RomaILSData& ilsData = RomaILSData(),
                 const RomaDirectoryAccess& auth = RomaDirectoryAccess());

    //----------------------------------------------------------------
    // Put a response.
    //
    // Passed response object provides options and data to be put.
    // The virtual RomaBeResponse::marshalResponse() method
    // is called before putting the response. The marshalResponse()
    // method can be overridden to marshal message data.
    //
    // Parameters:
    //   (in/out) response - a business element response to be put.
    //
    // Output attributes of the response are:
    //   - the theme
    //   - the message ID
    //   - the correlation ID
    //   - the time stamp
    //
    // Returns true on success.
    //----------------------------------------------------------------
    bool put(RomaBeResponse& response);

    //----------------------------------------------------------------
    // Get a request.
    //
    // Retrieves a request according to the specified get options.
    // Calls createRequest() to unmarshal the retrieved message.
    // The default implementation of the createRequest() creates an
    // instance of the RomaBeRequest. Overridden methods can return
    // custom requests derived from the RomaBeRequest.
    //
    // To safely cast returned pointer use ROMA_SAFE_PTR_CAST() macro.
    //
    // Parameters:
    //   (in) options - Roma get options
    //
    // Returns:
    //   non NULL - request created by new. User is responsible for
    //              deleting the returned object.
    //   NULL     - error or timeout occurred.
    //----------------------------------------------------------------
    RomaBeRequest* get();
    RomaBeRequest* get(RomaGetOptions& options);

    //----------------------------------------------------------------
    // Get a request.
    //
    // This form of the get command allows the application to supply
    // the message and tag buffers itself. If the call fails it will
    // return NULL. If the error code is KXCRC_BUFFER_TOO_SMALL then
    // the required buffer sizes are returned in the Length parameters
    // and the messsage is left on the queue so the operation can be
    // repeated with appropriate buffers. The Length parameters are
    // NOT updated unless the error code is KXCRC_BUFFER_TOO_SMALL.
    //----------------------------------------------------------------
    RomaBeRequest* get(RomaGetOptions& options,
                       void* msgBuffer,
                       int&  msgBufferLength,
                       void* tagBuffer,
                       int&  tagBufferLength);

    //----------------------------------------------------------------
    // An application management inline service can set an indicator
    // to inform a business element that it can shut down (if it
    // wishes). This method returns true if that inidcator was set
    // after the previous put or get operation.
    // Note, this indicator is only sent to business elements.
    //----------------------------------------------------------------
    bool isTermIndicator() const;

    //----------------------------------------------------------------
    // Terminate connection. Returns true on success.
    //----------------------------------------------------------------
    bool disconnect();

    //----------------------------------------------------------------
    // Returns true if connection established, false otherwise.
    //----------------------------------------------------------------
    bool isConnected() const;

    //----------------------------------------------------------------
    // Commit the local Roma transaction outstanding on the current
    // thread.
    //
    // This method doesn't rely on the syncpoint definition set for
    // the context - it always commits the "local" transaction.
    // Returns true on success.
    //----------------------------------------------------------------
    bool commitLocal();

    //----------------------------------------------------------------
    // Rollback the local Roma transaction outstanding on the
    // current thread.
    // Returns true on success.
    //----------------------------------------------------------------
    bool rollbackLocal();

    //----------------------------------------------------------------
    // Close theme.
    // Returns true on success. Null themes are ignored.
    //
    // Parameters:
    //   (in) theme - a theme to be closed.
    //----------------------------------------------------------------
    bool closeTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Delete named storage queue.
    // Returns true on success.
    //----------------------------------------------------------------
    bool deleteStorageQueue(const char* queueName);

    //----------------------------------------------------------------
    // Put Exception message.
    // Returns true on success.
    //
    // Parameters:
    //   (in/out) message - exception message
    //   (in) options - put options
    //----------------------------------------------------------------
    bool putException(RomaExceptionMessage& message,
        const RomaExceptionOptions& options = RomaExceptionOptions());

    //----------------------------------------------------------------
    // Report Error. Returns true on success, in which case parameter
    // 2 is updated to reflect the actual actions carried out and the
    // third parameter (if present) is also updated to reflect the
    // actual requested actions.
    //
    // Parameters:
    //   (in/out) message - the error description with (optionally)
    //            the message which caused the error
    //   (out)    the actions carried out - a set of ACTION_* flags.
    //   (in)     the report options
    //----------------------------------------------------------------
    bool reportError(RomaExceptionMessage& message,
                     KXCULONG& carriedOutActions,
                     const RomaReportOptions& options = RomaReportOptions());

    //----------------------------------------------------------------
    // Send alert. Returns true on success.
    //
    // Paramters:
    //   (in) alert
    //----------------------------------------------------------------
    bool sendAlert(const RomaLogInformation& alert);

    //----------------------------------------------------------------
    // Return any inline service data related to the initialization.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;


    //----------------------------------------------------------------
    // ERROR handling
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Returns true if any error information is available.
    //----------------------------------------------------------------
    bool anyError() const;

    //----------------------------------------------------------------
    // Returns any error information realting to the last Roma call.
    //----------------------------------------------------------------
    RomaError& error();

    //----------------------------------------------------------------
    // Clear the error, meaning the last Roma operation succeeded.
    //----------------------------------------------------------------
    void resetError();

    //----------------------------------------------------------------
    // Writes the specified information to the error log file whose
    // is defined by the KXCLOGFILENAME environment variable.
    //
    // If KXCLOGFILE is not set, an error is sent to the file
    // kxcuser.log (or DD:KXCULOG on MVS).
    //----------------------------------------------------------------
    bool writeLog(const RomaLogInformation& logInfo);

    //----------------------------------------------------------------
    // Writes the specified information to the system error log.
    // 
    // Paramters:
    //   (in) logInfo
    //----------------------------------------------------------------
    bool writeSysLog(const RomaLogInformation& logInfo);


    //----------------------------------------------------------------
    // EMS support.
    //----------------------------------------------------------------


    //----------------------------------------------------------------
    // Inserts the business service into the workflow of the theme
    // immediately after the current location in the workflow.
    //
    // Paramters:
    //      (out) newTheme - a new theme with the modified workflow
    //      (in)  curTheme - the existing theme
    //      (in)  bs       - the business service to insert
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool insertWorkflow(RomaTheme& newTheme,
                        const RomaTheme& curTheme,
                        const RomaBusinessService& bs,
                        bool close = true) const;

    //----------------------------------------------------------------
    // Replaces all remaining servers in the workflow with the
    // specified business service, leaving the client at the end of
    // the workflow.
    //
    // Paramters:
    //      (out) newTheme - a new theme with the modified workflow
    //      (in)  curTheme - the existing theme
    //      (in)  bs       - the business service to insert
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool replaceWorkflow(RomaTheme& newTheme,
                         const RomaTheme& curTheme,
                         const RomaBusinessService& bs,
                         bool close = true) const;

    //----------------------------------------------------------------
    // Sets the business flow information on the context. This
    // information will be included into the workflow information on
    // the next replaceWorkflow() or insertWorkflow() calls.
    //
    // Paramters:
    //      (in)  bf       - business flow information
    //----------------------------------------------------------------
    bool setBusinessFlow(const RomaBusinessFlow& bf) const;

    //----------------------------------------------------------------
    // Creates a new theme with exactly the same information as the
    // existing theme.
    //
    // Paramters:
    //      (out) newTheme - new theme copy
    //      (in)  curTheme - the source theme to copy
    //----------------------------------------------------------------
    bool copyTheme(RomaTheme& newTheme,
                   const RomaTheme& curTheme) const;

    //----------------------------------------------------------------
    // Merges the end-to-end and in-flight information with the
    // end-to-end and in-flight information that already exists for
    // the message and can be extracted with the queryTheme() method.
    //
    // Paramters:
    //      (out) newTheme - a new theme with the merged ETE
    //                       information
    //      (in)  curTheme - the existing theme
    //      (in)  ete      - ETE information
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool mergeEteInfo(RomaTheme& newTheme,
                      const RomaTheme& curTheme,
                      const RomaEteInfo& ete,
                      bool close = true) const;

    //----------------------------------------------------------------
    // Extracts detailed information from the specified theme.
    //
    // Paramters:
    //      (out) info    - extracted theme information
    //      (in)  theme   - the theme to query 
    //----------------------------------------------------------------
    bool queryTheme(RomaThemeInfo& info,
                    const RomaTheme& theme) const;

    //----------------------------------------------------------------
    // Creates a new theme from the existing theme and the
    // application-defined theme information.
    //
    // Paramters:
    //      (out) newTheme - a new theme
    //      (in)  curTheme - the existing theme
    //      (in)  info     - application-defined theme information
    //      (in)  options  - one or many Roma::ThemeInfo flags
    //                       indicating what parts of the theme
    //                       information to use
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool buildTheme(RomaTheme& newTheme,
                    const RomaTheme& curTheme,
                    const RomaThemeInfo& info,
                    unsigned int options,
                    bool close = true) const;

    //----------------------------------------------------------------
    // Extracts detailed message information. When CandleNet eBP
    // cannot process a message for any reason, it is rerouted to an
    // exception message queue, where you can, among other things,
    // examine its contents. This method enables an application to
    // process messages on the exception message queue.
    //
    // Note: if the application specifies Roma::INTMSG_THEME option
    //       to extract the original message theme, it is
    //       application's responsibility to close the extracted theme.
    //
    // Paramters:
    //      (out) info     - message information
    //      (in)  msg      - the message to be interpreted
    //      (in)  options  - zero or a combination of Roma::IntMsg
    //                       flags indicating additional parts of the
    //                       message information to extract
    //----------------------------------------------------------------
    bool interpretMessage(RomaMessageInfo& info,
                          const RomaBeRequest& msg,
                          unsigned int options = 0) const;


    //-----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //-----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;
    operator HKXC() const;
    HKXC operator *() const; // deprecated.


protected:
    //----------------------------------------------------------------
    // Unmarshals the retrieved request.
    //
    // This method is intended for overiding in derived classes.
    // Overidden method should assign "request" parameter with pointer
    // of the object of the RomaBeRequest or derived class, created by
    // new. If overridden method doesn't set "request" parameter or
    // set it to NULL then get() method sets KXORC_CREATE_REQUEST error.
    //
    // Parameters:
    //   (in)  baseRequest - retrieved message
    //   (out) newRequest  - marshaled request
    //
    // Returns:
    //   true  - success
    //   false - fail, get() method will set KXCORC_CREATE_REQUEST
    //           error but still return "request" to the caller.
    //----------------------------------------------------------------
    virtual bool createRequest(const RomaBeRequest& baseRequest,
                               RomaBeRequest*& newRequest);


    //-----------------------------------------------------------------
    // Pointer to the internal implementation.
    //-----------------------------------------------------------------
    RomaImplElement* m_impl;

private:
    //-----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //-----------------------------------------------------------------
    RomaBusinessElement(const RomaBusinessElement&);
    RomaBusinessElement& operator = (const RomaBusinessElement&);

    friend class RomaImplProxy;
};


#endif // __KXCOBE_HPP

