//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcocl.hpp,v 1.15 2003/12/11 18:30:04 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcocl.hpp,v $
// $Revision: 1.15 $
//--------------------------------------------------------------------
// The RomaClient public interface.
//--------------------------------------------------------------------

#ifndef __KXCOCL_HPP
#define __KXCOCL_HPP

#include <kxcocrs.hpp>
#include <kxcocrq.hpp>
#include <kxcosec.hpp>
#include <kxcosub.hpp>
#include <kxcopbl.hpp>
#include <kxcosbs.hpp>
#include <kxcoerr.hpp>
#include <kxcogto.hpp>
#include <kxcoils.hpp>
#include <kxcoem.hpp>
#include <kxcoli.hpp>
#include <kxcodac.hpp>
#include <kxcoems.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplClient;


//--------------------------------------------------------------------
// The RomaClient class.
//
// Represents the application's client-side use of Roma. 
// A client can:
//      * issue requests to a BS and receive responses
//      * publish and / or subscribe to specified Subjects
//      * commit and rollback local transactions
//      * work with the Roma EMS, write logs etc
//      * work with Roma directory data
//
// The RomaClient class can be sub-classed to differentiate between
// different response types. Such a sub-class specialises the
// createResponse() method to determine the response type and create
// the appropriate RomaClientResponse sub-class (refer to the
// Programmers Reference).
//--------------------------------------------------------------------
class KXCODLL RomaClient : public RomaTypeBase, public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Constructor.
    // Creates disconnected object. An application must call the
    // connect() method explicitly before any messages can be sent.
    //----------------------------------------------------------------
    RomaClient();

    //----------------------------------------------------------------
    // Constructor.
    // Establishes connection. See connect() for details.
    //----------------------------------------------------------------
    RomaClient(const char* name, 
               KXCULONG options = Roma::APP_NONE,
               const RomaSecurityDef& secDef = RomaSecurityDef(),
               const char* modelQueueName = NULL,
               const RomaILSData& ILSData = RomaILSData(),
               const RomaDirectoryAccess& auth = RomaDirectoryAccess());

    //----------------------------------------------------------------
    // Destructor.
    // Terminates connection.
    //----------------------------------------------------------------
    virtual ~RomaClient();

    //----------------------------------------------------------------
    // Establish client connection.
    //
    // Parameters:
    //   (in) name - client name
    //   (in) options - options, it is APP_NONE or any combination
    //        of the following flags:
    //        APP_EXCLUSIVE
    //        APP_MQSERIES_XACTION
    //        APP_NON_XACTIONAL
    //   (in) secDef - security definition
    //   (in) modelQueueName - a name of a model queue
    //   (in) ILSData - ILS data to hand off to this application
    //   (in) auth - directory authentication information
    //----------------------------------------------------------------
    bool connect(const char* name, 
                 KXCULONG options = Roma::APP_NONE,
                 const RomaSecurityDef& secDef = RomaSecurityDef(),
                 const char* modelQueueName = NULL,
                 const RomaILSData& ILSData = RomaILSData(),
                 const RomaDirectoryAccess& auth = RomaDirectoryAccess());

    //----------------------------------------------------------------
    // Put a request.
    //
    // The request object provides options and data to be sent.
    // The virtual RomaClientRequest::marshalRequest() method
    // is called before putting the request. The marshalRequest()
    // method can be overridden to marshal custom data.
    //
    // Parameters:
    //   (in/out) request
    //
    // Output attributes of the request are:
    //   - theme()
    //       set to new allocated theme if newTheme() == true or
    //       reset to null if theme closed by put operation
    //   - msgId()
    //       set to generated id if USE_MSGID is not specified
    //   - correlId()
    //   - timestamp()
    //       set to timestamp of the sent message
    //
    // Returns:
    //   false - error
    //   true  - success
    //----------------------------------------------------------------
    bool put(RomaClientRequest& request);

    //----------------------------------------------------------------
    // Analagous to a put, but the request is treated as a publication
    // on the specified subject.
    //
    // Parameters:
    //   publication defines service name and the data to be published
    //   sub specifies the subject for this publication.
    //
    // NOTE - This interface was provided in ebP v3.0, and has been 
    //        improved in version 3.2 to allow for retained 
    //        publications.  As part of this, the new RomaPublication
    //        object has been introduced, which combines RomaSubject
    //        and RomaClientRequest.  New applications should use
    //        the form of publish that takes a RomaPublication as its
    //        parameter.  This eBP v3.0 interface may be deprecated
    //        in a future version of eBP.
    //        
    //----------------------------------------------------------------
    bool publish(RomaClientRequest& publication, 
                 const RomaSubject& subject);

    //----------------------------------------------------------------
    // Analagous to a put, but the publication object defines data to
    // be published to all subscribers that have registered interest
    // in the subject of the publication.
    // 
    // Parameters:
    //   publication defines service name, subject and the data to be 
    //   published
    //
    //----------------------------------------------------------------
    bool publish(RomaPublication& publication);

    //----------------------------------------------------------------
    // Subscribe to the specified subject. No data is allowed in the
    // request message.
    // 
    // Parameters:
    //   req defines service name
    //   subject specifies the subject to which to subscribe
    //
    // NOTE - This interface was provided in ebP v3.0, and has been 
    //        improved in version 3.2 to allow for retained 
    //        publications.  As part of this, the new RomaSubscription
    //        object has been introduced, which combines RomaSubject
    //        and RomaClientRequest.  New applications should use
    //        the form of publish that takes a RomaSubscription as its
    //        parameter.  This eBP v3.0 interface may be deprecated
    //        in a future version of eBP.
    //----------------------------------------------------------------
    bool subscribe(RomaClientRequest& subscription, 
                   const RomaSubject& subject);

    //----------------------------------------------------------------
    // Subscribe to the subject defined by the subscription object.
    // 
    // Parameters:
    //   subscription specifies the service name and subject name
    //----------------------------------------------------------------
    bool subscribe(RomaSubscription& subscription);

    //----------------------------------------------------------------
    // Unsubscribe to the specified subject. No data is allowed in the
    // request message.
    //
    // Parameters:
    //   subscription - defines service name
    //   subject      - specifies the subject to unsubscribe from
    //
    // NOTE - This interface was provided in ebP v3.0, and has been 
    //        improved in version 3.2 to allow for retained 
    //        publications.  As part of this, the new RomaSubscription
    //        object has been introduced, which combines RomaSubject
    //        and RomaClientRequest.  New applications should use
    //        the form of publish that takes a RomaSubscription as its
    //        parameter.  This eBP v3.0 interface may be deprecated
    //        in a future version of eBP.
    //----------------------------------------------------------------
    bool unsubscribe(RomaClientRequest& subscription, 
                     const RomaSubject& subject);

    //----------------------------------------------------------------
    // Unsubscribe from the subject defined by the subscription object.
    // 
    // Parameters:
    //   subscription specifies the service name and subject name
    //
    //----------------------------------------------------------------
    bool unsubscribe(RomaSubscription& subscription);

    //----------------------------------------------------------------
    // Unsubscribe from all subjects. 
    // 
    // Parameters:
    //   subscription specifies the service name (and subject name)
    //   In this case, the subject is ignored and eBP unsubscribes
    //   from all subjects.
    //----------------------------------------------------------------
    bool unsubscribeAll(RomaSubscription& subscription);

    //----------------------------------------------------------------
    // Request all publications for the given subject since the last
    // requestUpdates() call.
    // 
    // Parameters:
    //   subscription specifies the service name and subject name
    //
    //----------------------------------------------------------------
    bool requestUpdates(RomaSubscription& subscription);

    //----------------------------------------------------------------
    // Delete a retained publication for the given subject 
    // 
    // Parameters:
    //   publication defines service name and subject to be deleted
    //
    //----------------------------------------------------------------
    bool deletePublication(RomaPublication& publication);

    //----------------------------------------------------------------
    // Get a response.
    //
    // Retrieves response according to specified get options,
    // calls createResponse() to unmarshal retrieved response and
    // returns created response object. The application is responsible
    // for deletion of the message.
    //
    // Default implementation of the createResponse() creates instance
    // of RomaClientResponse. Overridden methods can return
    // custom responses derived from RomaClientResponse.
    //
    // To safely cast returned pointer use ROMA_SAFE_PTR_CAST() macro.
    //
    // Parameters:
    //   (in) getOptions
    //
    // Returns:
    //   the response or NULL (if an error or timeout occurred)
    //----------------------------------------------------------------
    RomaClientResponse* get();
    RomaClientResponse* get(RomaGetOptions& options);

    //----------------------------------------------------------------
    // Get a response.
    //
    // This form of the get command allows the application to supply
    // the message and tag buffers itself. If the call fails it will
    // return NULL. If the error code is KXCRC_BUFFER_TOO_SMALL then
    // the required buffer sizes are returned in the Length parameters
    // and the messsage is left on the queue so the operation can be
    // repeated with appropriate buffers. The application is
    // responsible for deletion of the message.
    //----------------------------------------------------------------
    RomaClientResponse* get(RomaGetOptions& options,
                            void* msgBuffer,
                            int&  msgLength,
                            void* tagBuffer,
                            int&  tagLength);

    //----------------------------------------------------------------
    // Get any inline service data related to the initialization.
    // The application is responsible for deletion of the returned
    // object.
    //----------------------------------------------------------------
    RomaILSData* getILSData() const;

    //----------------------------------------------------------------
    // Terminate the connection.  Returns true on success.
    //----------------------------------------------------------------
    bool disconnect();

    //----------------------------------------------------------------
    // Returns true if connection exists, false otherwise.
    //----------------------------------------------------------------
    bool isConnected() const;

    //----------------------------------------------------------------
    // Commits the local Roma transaction outstanding on the current
    // thread.  Returns true on success.
    //
    // This method doesn't rely on the syncpoint definition set for
    // the context - it always commits the "local" transaction.
    //----------------------------------------------------------------
    bool commitLocal();

    //----------------------------------------------------------------
    // Rollback the local Roma transaction outstanding on the
    // current thread. Returns true on success.
    //----------------------------------------------------------------
    bool rollbackLocal();

    //----------------------------------------------------------------
    // Close theme. Return true on success. Null themes are ignored.
    //
    // Parameters:
    //   (in) theme - a theme to be closed
    //----------------------------------------------------------------
    bool closeTheme(const RomaTheme& theme);

    //----------------------------------------------------------------
    // Delete the named storage queue.
    //----------------------------------------------------------------
    bool deleteStorageQueue(const char* queueName);

    //----------------------------------------------------------------
    // Put Exception message. Returns true on success.
    //
    // Parameters:
    //   (in/out) message - exception message
    //   (in) options - put options
    //----------------------------------------------------------------
    bool putException(RomaExceptionMessage& message,
                      const RomaExceptionOptions& options = 
                        RomaExceptionOptions());

    //----------------------------------------------------------------
    // Report the error. Returns true on success, in which case the
    // second parameter is updated to reflect the actual actions
    // carried out.
    //
    // Parameters:
    //   (in/out) message - the error description with (optionally)
    //            the message which caused the error
    //   (out)    the actions carried out - a set of ACTION_* flags.
    //   (in)     the report options
    //----------------------------------------------------------------
    bool reportError(RomaExceptionMessage& message,
                     KXCULONG& carriedOutActions,
                     const RomaReportOptions& options = 
                        RomaReportOptions());

    //----------------------------------------------------------------
    // Send an alert. Returns true on success.
    //
    // Paramters:
    //   (in) alert
    //----------------------------------------------------------------
    bool sendAlert(const RomaLogInformation& alert);


    //----------------------------------------------------------------
    // ERROR handling
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Indicates whether any error information is available.
    //----------------------------------------------------------------
    bool anyError() const;

    //----------------------------------------------------------------
    // Get an error information realting to the last Roma call.
    //----------------------------------------------------------------
    RomaError& error();

    //----------------------------------------------------------------
    // Clear the error, meaning the last Roma operation succeeded.
    //----------------------------------------------------------------
    void resetError();

    //----------------------------------------------------------------
    // Writes the specified information to the error log file which
    // is defined by the KXCLOGFILENAME environment variable.
    //
    // If KXCLOGFILE is not set, an error is sent to the file
    // kxcuser.log (or DD:KXCULOG on MVS).
    //----------------------------------------------------------------
    bool writeLog(const RomaLogInformation& logInfo);

    //----------------------------------------------------------------
    // Writes the specified information to the system error log.
    // 
    // Paramters:
    //   (in) logInfo
    //----------------------------------------------------------------
    bool writeSysLog(const RomaLogInformation& logInfo);


    //----------------------------------------------------------------
    // EMS support.
    //----------------------------------------------------------------


    //----------------------------------------------------------------
    // Inserts the business service into the workflow of the theme
    // immediately after the current location in the workflow.
    //
    // Paramters:
    //      (out) newTheme - a new theme with the modified workflow
    //      (in)  curTheme - the existing theme
    //      (in)  bs       - the business service to insert
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool insertWorkflow(RomaTheme& newTheme,
                        const RomaTheme& curTheme,
                        const RomaBusinessService& bs,
                        bool close = true) const;

    //----------------------------------------------------------------
    // Replaces all remaining servers in the workflow with the
    // specified business service, leaving the client at the end of
    // the workflow.
    //
    // Paramters:
    //      (out) newTheme - a new theme with the modified workflow
    //      (in)  curTheme - the existing theme
    //      (in)  bs       - the business service to insert
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool replaceWorkflow(RomaTheme& newTheme,
                         const RomaTheme& curTheme,
                         const RomaBusinessService& bs,
                         bool close = true) const;

    //----------------------------------------------------------------
    // Sets the business flow information on the context. This
    // information will be included into the workflow information on
    // the next replaceWorkflow() or insertWorkflow() calls.
    //
    // Paramters:
    //      (in)  bf       - business flow information
    //----------------------------------------------------------------
    bool setBusinessFlow(const RomaBusinessFlow& bf) const;

    //----------------------------------------------------------------
    // Creates a new theme with exactly the same information as the
    // existing theme.
    //
    // Paramters:
    //      (out) newTheme - new theme copy
    //      (in)  curTheme - the source theme to copy
    //----------------------------------------------------------------
    bool copyTheme(RomaTheme& newTheme, const RomaTheme& curTheme) const;

    //----------------------------------------------------------------
    // Merges the end-to-end and in-flight information with the
    // end-to-end and in-flight information that already exists for
    // the message and can be extracted with the queryTheme() method.
    //
    // Paramters:
    //      (out) newTheme - a new theme with the merged ETE
    //                       information
    //      (in)  curTheme - the existing theme
    //      (in)  ete      - ETE information
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool mergeEteInfo(RomaTheme& newTheme,
                      const RomaTheme& curTheme,
                      const RomaEteInfo& ete,
                      bool close = true) const;

    //----------------------------------------------------------------
    // Extract detailed information from the specified theme.
    //
    // Paramters:
    //      (out) info    - extracted theme information
    //      (in)  theme   - the theme to query 
    //----------------------------------------------------------------
    bool queryTheme(RomaThemeInfo& info,
                    const RomaTheme& theme) const;

    //----------------------------------------------------------------
    // Creates a new theme from the existing theme and the
    // application-defined theme information.
    //
    // Paramters:
    //      (out) newTheme - a new theme
    //      (in)  curTheme - the existing theme
    //      (in)  info     - application-defined theme information
    //      (in)  options  - one or many Roma::ThemeInfo flags
    //                       indicating what parts of the theme
    //                       information to use
    //      (in)  close    - if true, closes the existing theme
    //----------------------------------------------------------------
    bool buildTheme(RomaTheme& newTheme,
                    const RomaTheme& curTheme,
                    const RomaThemeInfo& info,
                    unsigned int options,
                    bool close = true) const;

    //----------------------------------------------------------------
    // Extracts detailed message information. When CandleNet eBP
    // cannot process a message for any reason, it is rerouted to an
    // exception message queue, where you can, among other things,
    // examine its contents. This method enables an application to
    // process messages on the exception message queue.
    //
    // Note: if the application specifies Roma::INTMSG_THEME option
    //       to extract the original message theme, it is
    //       application's responsibility to close the extracted theme.
    //
    // Paramters:
    //      (out) info     - message information
    //      (in)  msg      - the message to be interpreted
    //      (in)  options  - zero or a combination of Roma::IntMsg
    //                       flags indicating additional parts of the
    //                       message information to extract
    //----------------------------------------------------------------
    bool interpretMessage(RomaMessageInfo& info,
                          const RomaClientResponse& msg,
                          unsigned int options = 0) const;


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    ROMA_DECLARE_TYPEID();
    virtual void trace(RomaTraceStream&) const;
    operator HKXC() const;    
    HKXC operator *() const; // deprecated.


protected:
    //----------------------------------------------------------------
    // Unmarshal the retrieved response.
    //
    // This method is intended for overriding in derived classes.
    // Overidden method should assign response parameter with pointer
    // of the object of the RomaBeRequest or derived class, created by
    // new. If overridden method doesn't set "response" parameter or
    // set it to 0 then get() method sets KXORC_CREATE_REQUEST error.
    //
    // Parameters:
    //   (out) response - marshaled response
    //
    // Returns:
    //   true  - success
    //   false - get() will set KXCORC_CREATE_RESPONSE
    //
    //----------------------------------------------------------------
    virtual bool createResponse(const RomaClientResponse& baseResponse,
                                RomaClientResponse*& newResponse);


    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplClient* m_impl;

private:
    //----------------------------------------------------------------
    // NOT IMPLEMENTED.
    //----------------------------------------------------------------
    RomaClient(const RomaClient&);
    RomaClient& operator = (const RomaClient&);

    friend class RomaImplProxy;
};


#endif // __KXCOCL_HPP

