//--------------------------------------------------------------------
//
// Copyright(C), 2002, an unpublished work by Candle Corporation.
// All rights reserved.  This material contains CONFIDENTIAL
// INFORMATION that is the property of Candle Corporation.
// Any unauthorized use, duplication or disclosure is prohibited.
//
//--------------------------------------------------------------------
// $Id: kxmsoapc.cpp,v 1.5 2004/10/04 17:06:27 e_vvb Exp $
// $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmsoapc.cpp,v $
// $Revision: 1.5 $
//--------------------------------------------------------------------
// SOAP sample client
//--------------------------------------------------------------------

#include <iostream.h>   // System header
#include <kxcocl.hpp>   // The Message Exchange API header.
#include <kxmmsg.hpp>   // Message Handling API Header.


/*********************************************************************
 * ########## CException ##########
 *********************************************************************/
class CException
{
public:
    CException( const char* er )
        : m_buffer( (void*)er, strlen(er) )
    {
    }

    const char* getErrorMessage() const 
    { 
        return (const char*)m_buffer.getBuffer();
    }
private:
    KxmDataBuffer m_buffer;
};


/*********************************************************************
 * ########## CRomaClient ##########
 *********************************************************************/
class CRomaClient
{
public:

    CRomaClient( 
        const char* clientName = "ClientGroup1", 
        const char* serverName = "TestService1"
        );

    void process( void* msg, size_t msgLen, KxmDataBuffer& resp );
    
private:
    void raise( RomaError& er );
    void raise( const char * er ) { throw CException(er); }

    RomaClient m_romaClient;
    const char* m_serviceName;   
};

//--------------------------------------------------------------------
// Initializes the client. Raises error if fail
//--------------------------------------------------------------------
CRomaClient::CRomaClient( const char* clientName, 
                          const char* serviceName )
:   m_serviceName( serviceName )
{
    cout << "Initializing the client...";
    cout.flush();

    m_romaClient.connect( clientName );

    if( m_romaClient.anyError() ) 
    {
        cout << "failed" << endl << endl;
        raise( m_romaClient.error() );
    }
    cout << "success" << endl << endl;
}

//--------------------------------------------------------------------
// Sends the request
//--------------------------------------------------------------------
void CRomaClient::process( void* msg, size_t msgLen, KxmDataBuffer& resp )
{
    // Prepares the request
    RomaClientRequest request( m_serviceName );
    request.setMsgBuffer( msg, msgLen );

    cout << "Sending the SOAP message to the server...";
    cout.flush();

    // Send the request
    if( !m_romaClient.put(request) ) 
    {
        cout << "failed" << endl << endl;
        raise( m_romaClient.error() );
    }
    cout << "success" << endl << endl;
    cout.flush();

    cout.write( (const char*)msg, msgLen );
    cout << endl;
    cout.flush();
  
    // Prepares options to get response
    RomaGetOptions options( request );
    options.setCloseTheme( true );
    options.setWaitInterval( 30000 );

    cout << "Waiting for a response... "; 
    cout.flush();

    // Attempt to get a response message.
    RomaClientResponse* pResponse = m_romaClient.get( options );
    if (!pResponse)
    {
        // If a response is NULL, then either an error occured or a
        // message does not appear on the queue during wait interval.
        cout << "failed" << endl << endl;

        if( m_romaClient.anyError() ) 
        {
            raise( m_romaClient.error() );
        }
        else
        {
            raise( "timed out" );
        }
    }
    cout << "success" << endl << endl;
    cout.flush();

    // Prints the received response
    const char* pData = (const char*)pResponse->msgBuffer();

    if( pData )
    {
        cout.write( pData, pResponse->msgDataLength() );
        resp = KxmDataBuffer( pResponse->msgBuffer(), 
                              pResponse->msgDataLength() );
    }
    else
    {
        resp = KxmDataBuffer();
    }

    cout << endl;
    cout.flush();

    // Cleanup the response
    delete pResponse;
}

//--------------------------------------------------------------------
// Composes and raises the CRomaClient error
//--------------------------------------------------------------------
void CRomaClient::raise( RomaError& er )
{
    const char* primaryError = er.errorText();
    const char* secondaryError = NULL;
    if ( er.status() == Roma::STATUS_BOTH )
    {
        secondaryError = er.errorText( Roma::SUBCLASS_SECONDARY );
    }

    char buffer[2048];
    strcpy( buffer, "\nPrimary error:  " );
    if( primaryError )
    {
        strcat( buffer, primaryError );
    }
    else
    {
        strcat(buffer, "<None>");
    }

    if( er.status() == Roma::STATUS_BOTH )
    {
        strcat( buffer, "\nSecondary error:  " );
        if( secondaryError )
        {
            strcat( buffer, secondaryError );
        }
        else
        {
            strcat(buffer, "<None>");
        }
    }

    strcat(buffer, "\n");
    raise( buffer );
}


/*********************************************************************
 * ########## CSoapClientMgr ##########
 *********************************************************************/
class CSoapClientMgr
{
public:

    void run();

private:

    void sample_1();
    void sample_quit();

    void raiseWrongResponse()
    {
        throw CException( "Wrong response received" );
    }

    CRomaClient client;
};


#define TOLOG_LINE_SEPARATOR \
    cout << "----------------------------------------------------" \
         << endl;

                                                      
//--------------------------------------------------------------------
// Creates the SOAP request 1
//--------------------------------------------------------------------
void CSoapClientMgr::run()
{
    sample_1();
    
    // Add more samples here
    // sample_2();

    // Request QUIT message for server to exit
    sample_quit();
}


//--------------------------------------------------------------------
// Creates the SOAP request 1
//--------------------------------------------------------------------
void CSoapClientMgr::sample_1()
{  
    TOLOG_LINE_SEPARATOR;
    cout << "Soap Sample 1" << endl << endl;
    cout << "Creating the SOAP message: [trade:GetLastTradePrice]" << endl;

    // Create the SOAP message to send
    KxmMessage msg( "SOAP-ENV", KxmMessage::MDR );
    msg.getField( "SOAP-ENV:Body.trade:GetLastTradePrice.Product").set( "wood" );

    // Read the just set field and dump it
    KxmField field = msg.getField("SOAP-ENV:Body.trade:GetLastTradePrice.Product");
    KxmStr value;
    field.get( value );
    cout << "  [" << field.getName().getNative() << "]: " << value.getNative() << endl;

    cout << endl;

    // Writes it to the buffer
    KxmDataBuffer buf;
    msg.write( buf , "NATIVE" );

    // Sends the request to server and waits for response
    KxmDataBuffer resp;
    client.process( buf.getBuffer(), buf.getDataLen(), resp );

    // Creates the response SOAP message
    cout << "Handling response: [trade:GetLastTradePriceResponse]" << endl;
    KxmMessage msgR( resp.getBuffer(), resp.getDataLen(), "SOAP-ENV", KxmMessage::MDR );
    
    KxmField field1 = msgR.getField("SOAP-ENV:Body.trade:GetLastTradePriceResponse.Company");
    if( field1.isEmpty() )
    {
        raiseWrongResponse();
        return;
    }
    
    // Check if proper response is received and prints it by fields
    field1.get( value );
    cout << "  [" << field1.getName().getNative() << "]: " << value.getNative() << endl;

    KxmField field2 = msgR.getField("SOAP-ENV:Body.trade:GetLastTradePriceResponse.Price");
    double price;
    field2.get( price );
    cout << "  [" << field2.getName().getNative() << "]: " << price << endl;

    KxmField field3 = msgR.getField("SOAP-ENV:Body.trade:GetLastTradePriceResponse.Price.@currency");
    field3.get( value );
    cout << "  [" << field3.getName().getNative() << "]: " << value.getNative() << endl;
    cout << endl;
}

//--------------------------------------------------------------------
// Creates the SOAP QUIT request
//--------------------------------------------------------------------
void CSoapClientMgr::sample_quit()
{
    TOLOG_LINE_SEPARATOR;
    cout << "Soap Sample QUIT" << endl << endl;
    cout << "Creating the SOAP message: [quit:QuitRequest]" << endl;

    // Create the SOAP message
    KxmMessage msg( "SOAP-ENV", KxmMessage::MDR );
    msg.getField( "SOAP-ENV:Body.quit:QuitRequest").set("");

    cout << endl;

    // Writes it to the buffer
    KxmDataBuffer buf;
    msg.write( buf , "NATIVE" );

    // Sends the request to server and waits for response
    KxmDataBuffer resp;
    client.process( buf.getBuffer(), buf.getDataLen(), resp );

    // Creates the response SOAP message
    cout << "Handling response: [quit:QuitResponse]" << endl;
    KxmMessage msgR( resp.getBuffer(), resp.getDataLen(), "SOAP-ENV", KxmMessage::MDR );
    
    // Check if proper response is received
    KxmField field = msgR.getField("SOAP-ENV:Body.quit:QuitResponse");
    if( field.isEmpty() )
    {
        raiseWrongResponse();
        return;
    }
    cout << endl;
}



/*********************************************************************
 * ########## main ##########
 *********************************************************************/
int main(int argc, char* argv[])
{
    try
    {
        CSoapClientMgr clientMgr;
        clientMgr.run();
    }
    catch( const CException& exc )
    {
        cout <<  exc.getErrorMessage() << endl;
        return EXIT_FAILURE;
    }
    catch( const KxmError& exc )
    {
        cout << exc.getErrorMessage() << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}
