//--------------------------------------------------------------------
//
// Copyright(C), 2002, an unpublished work by Candle Corporation.
// All rights reserved.  This material contains CONFIDENTIAL
// INFORMATION that is the property of Candle Corporation.
// Any unauthorized use, duplication or disclosure is prohibited.
//
//--------------------------------------------------------------------
// $Id: kxmsoaps.cpp,v 1.5 2004/10/04 17:05:56 e_vvb Exp $
// $Source: /romacvs/root/roma/src/kxm/kxmsmple/kxmsoaps.cpp,v $
// $Revision: 1.5 $
//--------------------------------------------------------------------
// SOAP sample server
//--------------------------------------------------------------------

#include <iostream.h>   // System headers.
#include <kxcobe.hpp>   // The Message Exchange API header.
#include <kxmmsg.hpp>   // Message Handling API Header.


/*********************************************************************
 * ########## CException ##########
 *********************************************************************/
class CException
{
public:
    CException( const char* er )
        : m_buffer( (void*)er, strlen(er) )
    {
    }

    const char* getErrorMessage() const 
    { 
        return (const char*)m_buffer.getBuffer();
    }
private:
    KxmDataBuffer m_buffer;
};


/*********************************************************************
 * ########## CRomaServer ##########
 *********************************************************************/

//--------------------------------------------------------------------
// Declares the server class based on Roma
//--------------------------------------------------------------------
class CRomaServer
{
public:

    CRomaServer( const char* elementName = "TestElement1" );
    ~CRomaServer();

    void putResponse( void* msg, size_t msgLen );
    KxmDataBuffer getRequest();
    
private:
    
    void raise( RomaError& er );
    void raise( const char* er ) { throw CException( er ); }

    const char* m_elementName;
    
    RomaBusinessElement m_romaServer;
    RomaBeRequest* m_romaBeRequest;
};

//--------------------------------------------------------------------
// Destructor. Cleanups the resources
//--------------------------------------------------------------------
CRomaServer::~CRomaServer()
{
    delete m_romaBeRequest;
}

//--------------------------------------------------------------------
// Initializes the server. Raises error if fail
//--------------------------------------------------------------------
CRomaServer::CRomaServer( const char* elementName )
:   m_elementName( elementName ),
    m_romaBeRequest(NULL)
{
    cout << "Initializing the server...";
    cout.flush();

    m_romaServer.connect( elementName );

    if( m_romaServer.anyError() ) 
    {
        cout << "failed" << endl << endl;
        raise( m_romaServer.error() );
    }
    cout << "success" << endl << endl;
}


//--------------------------------------------------------------------
// Gets the request
//--------------------------------------------------------------------
KxmDataBuffer CRomaServer::getRequest()
{
    // Sets the options
    RomaGetOptions options;
    options.setWaitInterval(120000);

    // Gets the request
    cout << "Waiting for a request...";
    cout.flush();

    delete m_romaBeRequest;
    m_romaBeRequest = m_romaServer.get( options );

    // Checks the request message
    if( !m_romaBeRequest )
    {
        // If the request is NULL, then either an error occured
        // or a message does not appear on the queue during wait
        // interval.
        cout << "failed" << endl << endl;
        if( m_romaServer.anyError() ) 
        {
            raise( m_romaServer.error() );
        }
        else
        {
            raise( "timed out" );
        }
    }

    cout << "success" << endl << endl;
    cout.flush();

    // Prints the requested date
    const char* pData = (const char*)m_romaBeRequest->msgBuffer();
    int length = m_romaBeRequest->msgDataLength();
    
    if (pData)
    {
        cout.write( pData, length );
        cout << endl;
    }
    cout.flush();

    return KxmDataBuffer( 
        m_romaBeRequest->msgBuffer(), 
        m_romaBeRequest->msgDataLength() 
        );
}

//--------------------------------------------------------------------
// Puts the response
//--------------------------------------------------------------------
void CRomaServer::putResponse( void* msg, size_t msgLen )
{
    // Prepares the request
    RomaBeResponse response( m_romaBeRequest );
    response.setMsgBuffer( msg, msgLen );
    
    delete m_romaBeRequest;
    m_romaBeRequest = NULL;

    // Put response
    cout << "Sending a response to the client...";
    cout.flush();

    if( !m_romaServer.put(response) ) 
    {
        // An error occured.
        cout << "failed" << endl << endl; 
        raise( m_romaServer.error() );
    }

    cout << "success" << endl << endl;
    cout.write( (const char*)msg, msgLen );
    cout << endl;
    cout.flush();
}

//--------------------------------------------------------------------
// Composes and raises the CRomaClient error
//--------------------------------------------------------------------
void CRomaServer::raise( RomaError& er )
{
    const char* primaryError = er.errorText();
    const char* secondaryError = NULL;
    if ( er.status() == Roma::STATUS_BOTH )
    {
        secondaryError = er.errorText( Roma::SUBCLASS_SECONDARY );
    }

    char buffer[2048];
    strcpy( buffer, "\nPrimary error:  " );
    if( primaryError )
    {
        strcat( buffer, primaryError );
    }
    else
    {
        strcat(buffer, "<None>");
    }

    if( er.status() == Roma::STATUS_BOTH )
    {
        strcat( buffer, "\nSecondary error:  " );
        if( secondaryError )
        {
            strcat( buffer, secondaryError );
        }
        else
        {
            strcat(buffer, "<None>");
        }
    }

    strcat(buffer, "\n");
    raise( buffer );
}


/*********************************************************************
 * ########## CSoapClientMgr ##########
 *********************************************************************/
class CSoapServerMgr
{
public:

    void run();

private:

    bool handle_1( const KxmDataBuffer& request );
    bool handle_quit( const KxmDataBuffer& request );

    void raiseWrongRequest()
    {
        throw CException( "Unsupported request received" );
    }

    CRomaServer server;
};

#define TOLOG_LINE_SEPARATOR \
    cout << "----------------------------------------------------" \
         << endl;

//--------------------------------------------------------------------
// Creates the SOAP request 1
//--------------------------------------------------------------------
void CSoapServerMgr::run()
{
    while( 1 )
    {
        TOLOG_LINE_SEPARATOR;

        // Gets the request
        KxmDataBuffer request = server.getRequest(); 
        
        if( handle_1( request ) ) continue;
        if( handle_quit( request ) ) break;
        
        raiseWrongRequest();
    }
}


//--------------------------------------------------------------------
// Handles the SOAP request 1
//--------------------------------------------------------------------
bool CSoapServerMgr::handle_1( const KxmDataBuffer& req )
{
    // Create the request message
    KxmMessage msg( req.getBuffer(), req.getDataLen(), "SOAP-ENV", KxmMessage::MDR );
    
    // Check if request 1 is received
    KxmField field = msg.getField("SOAP-ENV:Body.trade:GetLastTradePrice.Product");
    if( field.isEmpty() )
    {
        return false;
    }
    
    // Handling request
    cout << "Handling request: [trade:GetLastTradePrice]" << endl;
    KxmStr value;
    field.get( value );
    cout << "  [" << field.getName().getNative() << "]:" << value.getNative() << endl;

    cout << endl;

    // Creates the response message
    cout << "Creating the SOAP message: [trade:GetLastTradePriceResponse]" << endl;
    KxmMessage msgR( "SOAP-ENV", KxmMessage::MDR );

    msgR.getField( "SOAP-ENV:Body.trade:GetLastTradePriceResponse.Company").set( "Wood corp." );
    msgR.getField( "SOAP-ENV:Body.trade:GetLastTradePriceResponse.Price").set( 34.5 );
    msgR.getField( "SOAP-ENV:Body.trade:GetLastTradePriceResponse.Price.@currency").set( "USD" );
    
    KxmField f1 = msgR.getField( "SOAP-ENV:Body.trade:GetLastTradePriceResponse.Company");
    f1.get( value );
    cout << "  [" << f1.getName().getNative() << "]: " << value.getNative() << endl;

    KxmField f2 = msgR.getField( "SOAP-ENV:Body.trade:GetLastTradePriceResponse.Price");
    double price;
    f2.get( price );
    cout << "  [" << f2.getName().getNative() << "]: " << price << endl;

    KxmField f3 = msgR.getField( "SOAP-ENV:Body.trade:GetLastTradePriceResponse.Price.@currency");
    f3.get( value );
    cout << "  [" << f3.getName().getNative() << "]: " << value.getNative() << endl;

    cout << endl;

    // Writes the response to buffer
    KxmDataBuffer buf ;
    msgR.write( buf, "NATIVE" );

    // Send teh response
    server.putResponse( buf.getBuffer(), buf.getDataLen() );
    return true;
}


//--------------------------------------------------------------------
// Handles the SOAP request to QUIT
//--------------------------------------------------------------------
bool CSoapServerMgr::handle_quit( const KxmDataBuffer& req )
{
    // Create the request message
    KxmMessage msg( req.getBuffer(), req.getDataLen(), "SOAP-ENV", KxmMessage::MDR );
    
    // Check if QUIT request is received
    KxmField field = msg.getField("SOAP-ENV:Body.quit:QuitRequest");
    if( field.isEmpty() )
    {
        return false;
    }

    // Handling request
    cout << "Handling request: [quit:QuitRequest]" << endl;
    cout << endl;

    // Creates the response QUIT message
    cout << "Creating the response SOAP message: [quit:QuitResponse]" << endl;   
    KxmMessage msgR( "SOAP-ENV", KxmMessage::MDR );
    msgR.getField( "SOAP-ENV:Body.quit:QuitResponse").set("");

    cout << endl;

    // Writes the QUIT response to buffer
    KxmDataBuffer buf ;
    msgR.write( buf , "NATIVE" );

    // Send teh response
    server.putResponse( buf.getBuffer(), buf.getDataLen() );
    return true;
}



/*********************************************************************
 * ########## main ##########
 *********************************************************************/
int main(int argc, char* argv[])
{
    try
    {
        CSoapServerMgr serverMgr;
        serverMgr.run();
    }
    catch( const CException& exc )
    {
        cout <<  exc.getErrorMessage() << endl;
        return EXIT_FAILURE;
    }
    catch( const KxmError& exc )
    {
        cout << exc.getErrorMessage() << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}
