/********************************************************************/
/*                                                                  */
/* Module      : KXCEAR2.C                                          */
/*                                                                  */
/* Description : KXC Sample EAR functions                           */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Copyright Candle Corporation 1998                                */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Disclaimer:  This sample Error Analysis Routine is provided for  */
/*              information purposes only.  Candle neither makes    */
/*              any warranties or representations regarding this    */
/*              software, or its use.  This EAR is provided "as is" */
/*              and Candle expressly disclaims the warranties of    */
/*              merchantability and fitness for a particular        */
/*              purpose.                                            */
/*                                                                  */
/********************************************************************/
/*                                                                  */
/* Details:     This is another example Error Analysis Routine.     */
/*              This is the source for KXCEAR2, and is provided     */
/*              to show examples of a more complex EAR.             */
/*                                                                  */
/*              This module contains two levels of abstraction to   */
/*              provide a simple interface into the Error Analysis  */
/*              routines.  The first and lowest level handles the   */
/*              interface to eBP.  This consists of three           */
/*              functions; kxcEARInit, kxcEARTerm and kxcEARAnalyse.*/
/*                                                                  */
/*              kxcEARInit is called for when each eBP Context is   */
/*              initialized.  It allocates a structure to keep      */
/*              track of buffers the Analysis routine will allocate.*/
/*                                                                  */
/*              kxcEARTerm frees the buffers allocated by the EAR.  */
/*                                                                  */
/*              kxcEARAnalyse is called whenever an error occurs.   */
/*              The function determines the actions requested in the*/
/*              error profile and passes control to the second      */
/*              level of functions accordingly.                     */
/*                                                                  */
/*              The second level of functions manage the structures */
/*              passed between eBP and the EAR.  The basic flow     */
/*              of all these functions is:                          */
/*                - call the kxcear??? function containing the      */
/*                  logic which handles the errors.                 */
/*                - Allocate space in our context to store any new  */
/*                  data.                                           */
/*                - Copy the new data to the new buffers.           */
/*              Theses functions are named Do<EAR Action>.          */
/*                                                                  */
/*              A third level of functions is provided in the       */
/*              module kxcsacta.c.  These functions are passed the  */
/*              relevant details and return any new structures.  It */
/*              is in these functions that the error handling logic */
/*              resided.                                            */ 
/*                                                                  */
/*                                                                  */
/*              To build this EAR, modify the appropriate 'mkear'   */
/*              makefile for your platform and then issue:          */
/*                                                                  */
/*                On Unix      -  make -f mkear.mak                 */
/*                On NT/95/98  - nmake -f mkear.mak                 */
/*                                                                  */
/********************************************************************/

/********************************************************************/
/* Include standard compiler includes                               */
/********************************************************************/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/********************************************************************/
/* Include public header files                                      */
/********************************************************************/
#include <kxc.h>
#include <kxcxit.h>

/********************************************************************/
/* Include public macro header file                                 */
/********************************************************************/
#include <kxcmac.h>

/********************************************************************/
/* Include our own header files                                     */
/********************************************************************/
#include <kxcear2.h>
#include <kxcearac.h>

/********************************************************************/
/* Diagnostic tracing macros (disabled by default).                 */
/* To enable the tracing macros:                                    */
/*    1. ESTREAM must specify an open file descriptor.              */
/*    2. Uncomment the "#define  TRACE" statement.                  */
/*                                                                  */
/* Once tracing has been enabled, the tracing macros may be used    */
/* to conditionally write text to ESTREAM.  There are three such    */
/* macros:  TRACE0, TRACE1, and TRACE2.  Each requires as its       */
/* first argument a printf-style format string.  The macro name's   */
/* numeric suffix indicates the number of replaceable parameters    */
/* supported.  Examples:                                            */
/*                                                                  */
/*    TRACE0( "Program terminating\n." ) ;                          */
/*    TRACE1( "Return code = %i\n", iRC ) ;                         */
/*    TRACE2( "Service=%s, Return code = %i\n", Service, iRC ) ;    */
/*                                                                  */
/* If symbolic constant TRACE is undefined, TRACEn macro calls      */
/* are effectively no-ops.                                          */
/********************************************************************/
#define  ESTREAM          stderr
#define  FLUSH            fflush(ESTREAM);

#if !defined(TRACE)
#define TRACE                               /*  turn ON the trace   */
#endif

#ifdef   TRACE
#define  TRACE0(s)        fprintf(ESTREAM,s);       FLUSH
#define  TRACE1(s,p1)     fprintf(ESTREAM,s,p1);    FLUSH
#define  TRACE2(s,p1,p2)  fprintf(ESTREAM,s,p1,p2); FLUSH
#else
#define  TRACE0(s)        
#define  TRACE1(s,p1)     
#define  TRACE2(s,p1,p2)  
#endif

/********************************************************************/
/* Private prototypes                                               */
/********************************************************************/
static KXCBOOL DoAlert(HKXC        hKXC,
                       KXCCBACK    *pCBack,
                       PKXCEXCDTL  pExcDtl,
                       PKXCLOGDATA *ppLog);

static KXCBOOL DoLog(HKXC        hKXC,
                     KXCCBACK    *pCBack,
                     PKXCEXCDTL  pExcDtl,
                     PKXCLOGDATA *ppLog);

static KXCBOOL DoReset(HKXC           hKXC,
                       KXCCBACK       *pCBack,
                       PKXCEXCDTL     pExcDtl,
                       PKXCERRPROFILE *ppProfile,
                       PKXCEXCEPTION  *ppException,
                       PKXCULONG      pActions);

static KXCBOOL DoEMS(HKXC           hKXC,
                     KXCCBACK       *pCBack,
                     PKXCEXCDTL     pExcDtl,
                     PKXCEXCACT     pExcAct,
                     PKXCULONG      pActions);

static KXCBOOL DoEMSRaw(HKXC           hKXC,
                        KXCCBACK       *pCBack,
                        PKXCEXCDTL     pExcDtl,
                        PKXCEXCACT     pExcAct, 
                        PKXCULONG      pActions);

static KXCBOOL DoNotify (HKXC           hKXC,
                         KXCCBACK       *pCBack,
                         PKXCEXCDTL     pExcDtl,
                         PKXCEXCACT     pExcAct,
                         PKXCULONG      pActions);

/********************************************************************/
/* KXC EAR Function Pointer Table                                   */
/********************************************************************/
KXCERRFUNC kxcEARFuncs = { kxcEARInit,
                           kxcEARTerm,
                           kxcEARAnalyse};

/********************************************************************/
/* KXC Exit Definition                                              */
/********************************************************************/
KXCXIT EARExitDef = { KXCSIG_KXCXIT,
                      KXCXIT_VERSION_1, 
                      KXCXIT_TYPE_ERROR, 
                      KXCXIT_CURR_IFACE_LEVEL,
                      KXCXIT_FUNC_LEVEL_1,
                      KXCXIT_ERR_METHOD_DEF,
                      KXCXIT_CALLTYPE_CONFIG,
                      KXCXIT_ERR_NUMFUNCS,
                      &kxcEARFuncs};

/********************************************************************/
/* Define the Entry point for the in-line service                   */
/********************************************************************/
KXCXIT_DEFINEENTRY(EARExitDef)


/********************************************************************/
/* Private functions                                                */
/********************************************************************/

/********************************************************************/
/* DoAlert                                                          */
/*                                                                  */
/* Purpose    Called by EARAnalyse to handle the KXCEP_ACTION_ALERT */
/*            action.  Allocates the Alert structure in our context */
/*            if it isn't already there and calls kxcearFormatAlert */
/*            to format the alert message.                          */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            hKXC     - The eBP handle                             */
/*            pCBack   - The call back functions                    */
/*            pExcDtl  - The details for this error                 */
/*          Output                                                  */
/*            ppLog    - The KXCLOGDATA to be filled in             */
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - Alloc or kxcearFormatAlert failed             */
/*                                                                  */
/********************************************************************/
static KXCBOOL DoAlert(HKXC        hKXC,
                       KXCCBACK    *pCBack,
                       PKXCEXCDTL  pExcDtl,
                       PKXCLOGDATA *ppLog)
{
KXCBOOL  bSuccess    = TRUE;
PKXCCHAR pUserText   = NULL;
KXCULONG TextLen     = 0;

  TRACE0 ("DoAlert: Entering\n");
  /******************************************************************/
  /* Call the FormatAlert function                                  */
  /******************************************************************/
  TRACE0 ("DoAlert: call kxcearFormatAlert\n");
  bSuccess = kxcearFormatAlert (pExcDtl->pExceptionData,
                                pExcDtl->pErrorProfileData,
                                &pUserText, &TextLen);


  /******************************************************************/
  /* Allocate a LOGDATA Structure if this is the first time         */
  /******************************************************************/
  if ((bSuccess) && 
      (*ppLog == NULL))
  {
    TRACE0 ("DoAlert: New Log\n");
    bSuccess = (pCBack->Alloc)(hKXC,
                               KXCALLOC_CONTEXT,
                               sizeof(KXCLOGDATA),
                               (PKXCVOID *)ppLog);
    if (bSuccess)
    {
      kxcSetLogDefault(*ppLog);
    }
  }

  /******************************************************************/
  /* Free any previous text buffer - we don't know it's length      */
  /******************************************************************/
  if ((bSuccess) &&
      ((*ppLog)->pUserText != NULL))
  {
    TRACE0 ("DoAlert: Free Log Text\n");
    (pCBack->Free)(hKXC, (*ppLog)->pUserText);
    (*ppLog)->pUserText = NULL;
  }

  /******************************************************************/
  /* Allocate the eBP storage for the message                       */
  /******************************************************************/
  if ((bSuccess) &&
      ((*ppLog)->pUserText == NULL))
  {
    TRACE0 ("DoAlert: Alloc new usertext\n");
    bSuccess = (pCBack->Alloc)(hKXC,
                               KXCALLOC_CONTEXT,
                               TextLen + 1,
                               (PKXCVOID *)&(*ppLog)->pUserText);
  }

  /******************************************************************/
  /* Copy the formatted buffer                                      */
  /******************************************************************/
  if ((bSuccess) && 
      pUserText != NULL)
  {
    TRACE0 ("DoAlert: Copy buffer\n");
    memcpy ((*ppLog)->pUserText, pUserText, TextLen);
    (*ppLog)->pUserText [TextLen] = '\0';
  }
  
  /******************************************************************/
  /* Free the temporary buffer                                      */
  /******************************************************************/
  if (pUserText != NULL)
  {
    TRACE0 ("DoAlert: Free temp buffer\n");
    free (pUserText);
    pUserText = NULL;
  }

  TRACE1 ("DoAlert: Return %s\n", (bSuccess ? "success" : "failure"));
  return (bSuccess);
}

/********************************************************************/
/* DoLog                                                            */
/*                                                                  */
/* Purpose    Called by EARAnalyse to handle the KXCEP_ACTION_LOG   */
/*            action.  Allocates the Log structure in our context   */
/*            if it isn't already there and calls kxcearFormatLog   */
/*            to format the alert message.                          */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            hKXC     - The eBP handle                             */
/*            pCBack   - The call back functions                    */
/*            pExcDtl  - The details for this error                 */
/*          Output                                                  */
/*            ppLog    - The KXCLOGDATA to be filled in             */
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - Alloc or kxcearFormatLog failed               */
/*                                                                  */
/********************************************************************/
static KXCBOOL DoLog(HKXC        hKXC,
                     KXCCBACK    *pCBack,
                     PKXCEXCDTL  pExcDtl,
                     PKXCLOGDATA *ppLog)
{
KXCBOOL  bSuccess    = TRUE;
PKXCCHAR pUserText   = NULL;
KXCULONG TextLen     = 0;
PKXCVOID pBuffer     = NULL;
KXCULONG BufLen      = 0;
KXCULONG Severity    = KXCLOGSEVERITY_ERROR;
KXCULONG Format      = KXCLOGFORMAT_HEXDUMP;


  TRACE0 ("DoLog: Entering\n");
  /******************************************************************/
  /* Call the FormatLog function                                    */
  /******************************************************************/
  TRACE0 ("DoLog: Format Log\n");
  bSuccess = kxcearFormatLog (pExcDtl->pExceptionData,
                              pExcDtl->pErrorProfileData,
                              &Severity,  &Format,
                              &pUserText, &TextLen,
                              &pBuffer,   &BufLen);

  /******************************************************************/
  /* Allocate a LOGDATA Structure if this is the first time         */
  /******************************************************************/
  if ((bSuccess) &&
      (*ppLog == NULL))
  {
    TRACE0 ("DoLog: New Alert log\n");
    bSuccess = (pCBack->Alloc)(hKXC,
                               KXCALLOC_CONTEXT,
                               sizeof(KXCLOGDATA),
                               (PKXCVOID *)ppLog);
    if (bSuccess)
    {
      kxcSetLogDefault(*ppLog);
    }
  }

  /******************************************************************/
  /* Assign the format and the severity                             */
  /******************************************************************/
  if (bSuccess)
  {
    (*ppLog)->Severity = Severity;
    (*ppLog)->Format   = Format;
  }    

  /******************************************************************/
  /* Free any previous text buffer, we don't know it's length       */
  /******************************************************************/
  if ((bSuccess) &&
      ((*ppLog)->pUserText != NULL))
  {
    TRACE0 ("DoLog: Free old text\n");
    (pCBack->Free)(hKXC, (*ppLog)->pUserText);
    (*ppLog)->pUserText = NULL;
  }

  /******************************************************************/
  /* Allocate storage for ppLog->pUserText                          */
  /******************************************************************/
  if ((bSuccess) &&
      ((*ppLog)->pUserText == NULL))
  {
    TRACE0 ("DoLog: New user text\n");
    bSuccess = (pCBack->Alloc)(hKXC,
                               KXCALLOC_CONTEXT,
                               TextLen + 1,
                               (PKXCVOID *)&(*ppLog)->pUserText);
  }

  /******************************************************************/
  /* Copy the pUserText formatted buffer                            */
  /******************************************************************/
  if ((bSuccess) && 
      pUserText != NULL)
  {
    TRACE0 ("DoLog: Copy User Text\n"); 
    memcpy ((*ppLog)->pUserText, pUserText, TextLen);
    (*ppLog)->pUserText [TextLen] = '\0';
  }
  
  /******************************************************************/
  /* Allocate storage for pBuf (if it isn't already big enough)     */
  /******************************************************************/
  if ((bSuccess) &&
      (pBuffer != NULL))
  {
    TRACE0 ("DoLog: Sort out buffer size\n");
    /****************************************************************/
    /* If the buffer is there but too small free it                 */
    /****************************************************************/
    if (((*ppLog)->DataBuf.pBuffer != NULL) &&
        ((*ppLog)->DataBuf.BufLen < BufLen))
    {
      TRACE0 ("DoLog: buffer too small\n");
      (pCBack->Free)(hKXC, (*ppLog)->DataBuf.pBuffer);
      (*ppLog)->DataBuf.pBuffer = NULL;
      (*ppLog)->DataBuf.BufLen = 0;
    }
    
    /****************************************************************/
    /* Allocate a new buffer (it's been free or was never there)    */
    /****************************************************************/
    if ((bSuccess) &&
        (*ppLog)->DataBuf.pBuffer == NULL)
    {
      TRACE0 ("DoLog: New buffer allocated\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 BufLen,
                     (PKXCVOID *)(&(*ppLog)->DataBuf.pBuffer));
      if (bSuccess)
      {
        (*ppLog)->DataBuf.BufLen = BufLen;
      }
    }

    /****************************************************************/
    /* Take a copy of the buffer returned from earFormatLog         */
    /****************************************************************/
    if (bSuccess)
    {
      TRACE0 ("DoLog: Copy buffer\n");
      memcpy((*ppLog)->DataBuf.pBuffer, pBuffer, BufLen);
      (*ppLog)->DataLen = BufLen;
    }
  }
  
  /******************************************************************/
  /* Free the temporary buffers                                     */
  /******************************************************************/
  if (pUserText != NULL)
  {
    TRACE0 ("DoLog: Free temp user text\n");
    free (pUserText);
    pUserText = NULL;
  }

  if (pBuffer != NULL)
  {
    TRACE0 ("DoLog: Free temp buffer\n");
    free (pBuffer);
    pBuffer = NULL;
  }

  TRACE1 ("DoLog: Return %s\n", (bSuccess ? "success" : "failure"));
  return (bSuccess);
}

/********************************************************************/
/* DoReset                                                          */
/*                                                                  */
/* Purpose    Called by EARAnalyse to handle the KXCEP_ACTION_RESET */
/*            action.  Calls the kxcearResetError function to       */
/*            fill in the Error Profile and Exception structures    */
/*            as required.                                          */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            hKXC           - The eBP handle                       */
/*            pCBack         - The call back functions              */
/*            pExcDtl        - The details for this error           */
/*          Output                                                  */
/*            ppProfile      - The KXCERRPROFILE to be filled in    */
/*            ppException    - The KXCEXCEPTION to be filled in     */
/*            pActions       - Indicates what structs have changed  */  
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - Alloc or kxcearResetError failed              */
/*                                                                  */
/********************************************************************/
static KXCBOOL DoReset(HKXC           hKXC,
                       KXCCBACK       *pCBack,
                       PKXCEXCDTL     pExcDtl,
                       PKXCERRPROFILE *ppProfile,
                       PKXCEXCEPTION  *ppException,
                       PKXCULONG      pActions)
{
KXCBOOL        bSuccess     = TRUE;
KXCULONG       Mods         = KXCSERVACT_NONE;
KXCERRPROFILE  Profile      = { KXCEPROF_DEFAULT };
KXCEXCEPTION   Exception    = { KXCEXCEPTION_DEFAULT };
  
  TRACE0 ("DoReset: Entering\n");
  /******************************************************************/
  /* TODO:Set the profile to the current settings - not for now.    */
  /******************************************************************/

  /******************************************************************/
  /* Call function to set the Profile and Exception                 */
  /******************************************************************/
  TRACE0 ("DoReset: ResetError\n");
  bSuccess = kxcearResetError(pExcDtl,
                              &Profile,
                              &Exception,
                              &Mods);

  /******************************************************************/
  /* If OK and the Profile was modified                             */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWERRPROFILE))
  {
    TRACE0 ("DoReset: New Profile created\n");
    /****************************************************************/
    /* Allocate a new profile if there isn't one                    */
    /****************************************************************/
    if (*ppProfile == NULL)
    {
      TRACE0 ("DoReset: New profile allocated\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 sizeof(KXCERRPROFILE),
                                 (PKXCVOID *)ppProfile);
    }
    
    /****************************************************************/
    /* Copy in the returned profile and tell caller it's changed    */
    /****************************************************************/
    if (bSuccess)
    {
      TRACE0 ("DoReset: Profile copied\n");
      **ppProfile = Profile;
      *pActions |= KXCEXCACT_NEWERRPROFILE;
    }
  }

  /******************************************************************/
  /* If OK and the Exception struct was modified                    */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEXCEPTION))
  {
    TRACE0 ("DoReset: New Exception\n");
    /****************************************************************/
    /* Allocate a new exception there isn't one                     */
    /****************************************************************/
    if (*ppException == NULL)
    {
      TRACE0 ("DoReset: New Exception allocated\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 sizeof(KXCEXCEPTION),
                                 (PKXCVOID *)ppException);
    }
    
    /****************************************************************/
    /* Copy in the returned exception and tell caller it's changed  */
    /****************************************************************/
    if (bSuccess)
    {
      TRACE0 ("DoReset: New exception copied\n");
      **ppException = Exception;
      *pActions |= KXCEXCACT_NEWEXCEPTION;
    }
  }

  TRACE1 ("DoReset: Return %s\n", (bSuccess ? "success" : "failure"));
  return (bSuccess);
}

/********************************************************************/
/* DoEMS                                                            */
/*                                                                  */
/* Purpose    Called by EARAnalyse to handle the KXCEP_ACTION_EMS   */
/*            action.  Calls the kxcearModifyMsgForEMS function to  */
/*            format a new message or tag buffer before it is sent  */
/*            to the EMS.                                           */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            hKXC     - The eBP handle                             */
/*            pCBack   - The call back functions                    */
/*            pExcDtl  - The details for this error                 */
/*          Output                                                  */
/*            pExcAct    - The action details                       */
/*            pActions   - Indicates what structs have changed      */  
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - Alloc or kxcearModifyMsgForEMS failed         */
/*                                                                  */
/* Warning                                                          */
/*            The DoNotify function writes and new message/tag      */
/*            buffers to the same place as DoEMS.  If both EMS and  */
/*            Notify are actions and they both create a new message */
/*            /tag then one message/tag will be lost.               */
/*                                                                  */
/********************************************************************/
static KXCBOOL DoEMS(HKXC           hKXC,
                     KXCCBACK       *pCBack,
                     PKXCEXCDTL     pExcDtl,
                     PKXCEXCACT     pExcAct, 
                     PKXCULONG      pActions)
{
KXCBOOL        bSuccess   = TRUE;
KXCULONG       Mods       = KXCSERVACT_NONE;
PKXCVOID       pMsg       = NULL;
KXCULONG       MsgLen     = 0;
PKXCVOID       pTag       = NULL;
KXCULONG       TagLen     = 0;
PKXCBUFFER     pBuffer;
PKXCEXCMSG     pExcMsg    = pExcDtl->pMessage;

  TRACE0 ("DoEMS: Entering\n");

  /******************************************************************/
  /* Call function to set the Message and Tag buffers               */
  /******************************************************************/
  TRACE0 ("DoEMS: Calling ModifyForEMS\n");
  bSuccess = kxcearModifyMsgForEMS(pExcDtl,
                                   &pMsg, &MsgLen,
                                   &pTag, &TagLen,
                                   &Mods);
  
  /******************************************************************/
  /* If OK and the Msg length was modified                          */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEMSMSGLEN))
  {
    TRACE0 ("DoEMS: New Msg Len\n");
    pExcAct->EMSMsgLen = MsgLen;
    *pActions |= KXCEXCACT_NEWEMSMSGLEN;
  }
  else
  {
    MsgLen = pExcMsg->EMSMessageLen;
  }

  /******************************************************************/
  /* If OK and the Message Buffer was modified                      */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEMSMSG))
  {
    TRACE0 ("DoEMS: New message buffer\n");
    /****************************************************************/
    /* If the buffer is there but too small free it                 */
    /****************************************************************/  
    pBuffer = &pExcAct->EMSMsg;
    if ((pBuffer->pBuffer != NULL) &&
        (pBuffer->BufLen < MsgLen))
    {
      TRACE0 ("DoEMS: Old buffer too small\n");
      (pCBack->Free)(hKXC, pBuffer->pBuffer);
      pBuffer->pBuffer = NULL;
      pBuffer->BufLen = 0;
    }
    
    /****************************************************************/
    /* Allocate a new buffer if needed                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pBuffer->pBuffer == NULL) &&
        (MsgLen > 0))
    {
      TRACE0 ("DoEMS: New buffer allocated\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 MsgLen,
                                 (PKXCVOID *)&pBuffer->pBuffer);
      if (bSuccess)
      {
        pBuffer->BufLen = MsgLen;
      }
    }

    /****************************************************************/
    /* Take a copy of the buffer returned from earModifyMsgForEMS   */
    /****************************************************************/
    if (bSuccess)
    {
      if (MsgLen > 0)
      {
        TRACE0 ("DoEMS: Copy message\n");
        memcpy(pBuffer->pBuffer, 
               pMsg, 
               MsgLen);
      }
      *pActions |= KXCEXCACT_NEWEMSMSG;
    }
  }

  /******************************************************************/
  /* If OK and the tag length was modified                          */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEMSTAGLEN))
  {
    TRACE0 ("DoEMS: New Tag Len\n");
    pExcAct->EMSTagLen = TagLen;
    *pActions |= KXCEXCACT_NEWEMSTAGLEN;
  }
  else
  {
    TagLen = pExcMsg->EMSTagLen;
  }

  /******************************************************************/
  /* If OK and the Tag buffer was modified                          */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEMSTAG))
  {
    TRACE0 ("DoEMS: New Tag Buffer\n");
    /****************************************************************/
    /* If the buffer is there but too small free it                 */
    /****************************************************************/  
    pBuffer = &pExcAct->EMSTag;
    if ((pBuffer->pBuffer != NULL) &&
        (pBuffer->BufLen < TagLen))
    {
      TRACE0 ("DoEMS: Old buffer too small\n");
      (pCBack->Free)(hKXC, pBuffer->pBuffer);
      pBuffer->pBuffer = NULL;
      pBuffer->BufLen = 0;
    }
    
    /****************************************************************/
    /* Allocate a new buffer if needed                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pBuffer->pBuffer == NULL) &&
        (TagLen > 0))
    {
      TRACE0 ("DoEMS: Alloc new buffer\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 TagLen,
                                 (PKXCVOID *)&pBuffer->pBuffer);
      if (bSuccess)
      {
        pBuffer->BufLen = TagLen;
      }
    }

    /****************************************************************/
    /* Take a copy of the buffer returned from earModifyMsgForEMS   */
    /****************************************************************/
    if (bSuccess)
    {
      if (TagLen > 0)
      {
        TRACE0 ("DoEMS: Copy tag buffer\n");
        memcpy(pBuffer->pBuffer, 
               pTag, 
               TagLen);
      }
      *pActions |= KXCEXCACT_NEWEMSTAG;
    }
  }
  
  /******************************************************************/
  /* Finally, if earModifyForEMS allocated buffers free them        */
  /******************************************************************/
  if (pMsg != NULL)
  {
    TRACE0 ("DoEMS: free temp msg buffer\n");
    free (pMsg);
  }

  if (pTag != NULL)
  {
    TRACE0 ("DoEMS: free temp tag buffer\n");
    free (pTag);
  }

  TRACE1 ("DoEMS: Return %s\n", (bSuccess ? "success" : "failure"));
  return (bSuccess);
}


/********************************************************************/
/* DoEMSRaw                                                         */
/*                                                                  */
/* Purpose    Called by EARAnalyse to handle the KXCEP_ACTION_EMSRAW*/
/*            action.  Calls the kxcearCreateEMSRaw function to     */
/*            format a new EMSRaw message before it is sent to the  */
/*            EMS.                                                  */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            hKXC     - The eBP handle                             */
/*            pCBack   - The call back functions                    */
/*            pExcDtl  - The details for this error                 */
/*          Output                                                  */
/*            pExcAct    - The action details                       */
/*            pActions   - Indicates what structs have changed      */
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - Alloc or kxcearCreateEMSRaw failed            */
/*                                                                  */
/********************************************************************/
static KXCBOOL DoEMSRaw(HKXC           hKXC,
                        KXCCBACK       *pCBack,
                        PKXCEXCDTL     pExcDtl,
                        PKXCEXCACT     pExcAct, 
                        PKXCULONG      pActions)
{
KXCBOOL        bSuccess   = TRUE;
KXCULONG       Mods       = KXCSERVACT_NONE;
PKXCVOID       pEMSRaw    = NULL;
KXCULONG       EMSRawLen  = 0;
PKXCBUFFER     pBuffer;

  TRACE0 ("DoEMSRaw: Entering\n");

  /******************************************************************/
  /* Call function to create the raw EMS message                    */
  /******************************************************************/
  TRACE0 ("DoEMSRaw: Calling CreateEMSRaw\n");
  bSuccess = kxcearCreateEMSRaw(pExcDtl,
                                &pEMSRaw, &EMSRawLen,
                                &Mods);
  
  /******************************************************************/
  /* If OK and the length was modified - we always expect this      */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEMSRAWLEN))
  {
    TRACE0 ("DoEMSRaw: New EMSRaw Len\n");
    pExcAct->RawMsgLen = EMSRawLen;
    *pActions |= KXCEXCACT_NEWEMSRAWLEN;
  }

  /******************************************************************/
  /* If OK and the Message Buffer was modified                      */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWEMSRAW))
  {
    TRACE0 ("DoEMSRaw: New EMS Raw buffer\n");
    /****************************************************************/
    /* If the buffer is there but too small free it                 */
    /****************************************************************/  
    pBuffer = &pExcAct->RawMessage;
    if ((pBuffer->pBuffer != NULL) &&
        (pBuffer->BufLen < EMSRawLen))
    {
      TRACE0 ("DoEMSRaw: Old buffer too small\n");
      (pCBack->Free)(hKXC, pBuffer->pBuffer);
      pBuffer->pBuffer = NULL;
      pBuffer->BufLen = 0;
    }
    
    /****************************************************************/
    /* Allocate a new buffer if needed                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pBuffer->pBuffer == NULL) &&
        (EMSRawLen > 0))
    {
      TRACE0 ("DoEMSRaw: New buffer allocated\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 EMSRawLen,
                                 (PKXCVOID *)&pBuffer->pBuffer);
      if (bSuccess)
      {
        pBuffer->BufLen = EMSRawLen;
      }
    }

    /****************************************************************/
    /* Take a copy of the buffer returned from kxcearCreateEMSRaw   */
    /****************************************************************/
    if (bSuccess)
    {
      if (EMSRawLen > 0)
      {
        TRACE0 ("DoEMSRaw: Copy message\n");
        memcpy(pBuffer->pBuffer, 
               pEMSRaw, 
               EMSRawLen);
      }
      *pActions |= KXCEXCACT_NEWEMSRAW;
    }
  }

  /******************************************************************/
  /* Finally, if earCreateEMSRaw allocated buffers free them        */
  /******************************************************************/
  if (pEMSRaw != NULL)
  {
    TRACE0 ("DoEMSRaw: free temp msg buffer\n");
    free (pEMSRaw);
  }

  TRACE1 ("DoEMSRaw: Return %s\n", (bSuccess ? "success" : "failure"));
  return (bSuccess);
}

/********************************************************************/
/* DoNotify                                                         */
/*                                                                  */
/* Purpose    Called by EARAnalyse to handle the KXCEP_ACTION_NOTIFY*/
/*            action.  Calls the kxcearModifyMsgForNotify function  */
/*            to format a new message or tag buffer before it is    */
/*            sent as a notify message.                             */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            hKXC     - The eBP handle                             */
/*            pCBack   - The call back functions                    */
/*            pExcDtl  - The details for this error                 */
/*          Output                                                  */
/*            pExcAct    - The action details                       */
/*            pActions   - Indicates what structs have changed      */  
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - Alloc or kxcearModifyMsgForNotify failed      */
/*                                                                  */
/* Warning                                                          */
/*            The DoNotify function writes and new message/tag      */
/*            buffers to the same place as DoEMS.  If both EMS and  */
/*            Notify are actions and they both create a new message */
/*            /tag then one message/tag will be lost.               */
/*                                                                  */
/********************************************************************/
static KXCBOOL DoNotify(HKXC           hKXC,
                        KXCCBACK       *pCBack,
                        PKXCEXCDTL     pExcDtl,
                        PKXCEXCACT     pExcAct, 
                        PKXCULONG      pActions)
{
KXCBOOL        bSuccess   = TRUE;
KXCULONG       Mods       = KXCSERVACT_NONE;
PKXCVOID       pMsg       = NULL;
KXCULONG       MsgLen     = 0;
PKXCVOID       pTag       = NULL;
KXCULONG       TagLen     = 0;
PKXCBUFFER     pBuffer    = NULL;
PKXCEXCMSG     pExcMsg    = pExcDtl->pMessage;

  TRACE0 ("DoNotify: Entering\n");
  /******************************************************************/
  /* Call function to set the Message and Tag buffers               */
  /******************************************************************/
  TRACE0 ("DoNotify: Calling MsgForNotify\n");
  bSuccess = kxcearModifyMsgForNotify(pExcDtl,
                                      &pMsg, &MsgLen,
                                      &pTag, &TagLen,
                                      &Mods);
  
  /******************************************************************/
  /* If OK and the Msg length was modified                          */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWNOTIFYMSGLEN))
  {
    TRACE0 ("DoNotify: New Msg Len\n");
    pExcAct->NotifyMsgLen = MsgLen;
    *pActions |= KXCEXCACT_NEWNOTIFYMSGLEN;
  }
  else
  {
    MsgLen = pExcMsg->NotifyMessageLen;
  }

  /******************************************************************/
  /* If OK and the Message Buffer was modified                      */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWNOTIFYMSG))
  {
    TRACE0 ("DoNotify: New message buffer\n");
    /****************************************************************/
    /* If the buffer is there but too small free it                 */
    /****************************************************************/  
    pBuffer = &pExcAct->NotifyMsg;
    if ((pBuffer->pBuffer != NULL) &&
        (pBuffer->BufLen < MsgLen))
    {
      TRACE0 ("DoNotify: Old buffer too small\n");
      (pCBack->Free)(hKXC, pBuffer->pBuffer);
      pBuffer->pBuffer = NULL;
      pBuffer->BufLen = 0;
    }
    
    /****************************************************************/
    /* Allocate a new buffer if needed                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pBuffer->pBuffer == NULL) &&
        (MsgLen > 0))
    {
      TRACE0 ("DoNotify: New buffer allocated\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 MsgLen,
                                 (PKXCVOID *)&pBuffer->pBuffer);
      if (bSuccess)
      {
        pBuffer->BufLen = MsgLen;
      }
    }

    /****************************************************************/
    /* Take a copy of the buffer returned from earModifyMsgForEMS   */
    /****************************************************************/
    if (bSuccess)
    {
      if (MsgLen > 0)
      {
        TRACE0 ("DoNotify: Copy message\n");
        memcpy(pBuffer->pBuffer, 
               pMsg, 
               MsgLen);
      }
      *pActions |= KXCEXCACT_NEWNOTIFYMSG;
    }
  }

  /******************************************************************/
  /* If OK and the tag length was modified                          */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWNOTIFYTAGLEN))
  {
    TRACE0 ("DoNotify: New Tag Len\n");
    pExcAct->NotifyTagLen = TagLen;
    *pActions |= KXCEXCACT_NEWNOTIFYTAGLEN;
  }
  else
  {
    TagLen = pExcMsg->NotifyTagLen;
  }

  /******************************************************************/
  /* If OK and the Tag buffer was modified                          */
  /******************************************************************/
  if ((bSuccess) &&
      (Mods & KXCEXCACT_NEWNOTIFYTAG))
  {
    TRACE0 ("DoNotify: New Tag Buffer\n");
    /****************************************************************/
    /* If the buffer is there but too small free it                 */
    /****************************************************************/  
    pBuffer = &pExcAct->NotifyTag;
    if ((pBuffer->pBuffer != NULL) &&
        (pBuffer->BufLen < TagLen))
    {
      TRACE0 ("DoNotify: Old buffer too small\n");
      (pCBack->Free)(hKXC, pBuffer->pBuffer);
      pBuffer->pBuffer = NULL;
      pBuffer->BufLen = 0;
    }
    
    /****************************************************************/
    /* Allocate a new buffer if needed                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pBuffer->pBuffer == NULL) &&
        (TagLen > 0))
    {
      TRACE0 ("DoNotify: Alloc new buffer\n");
      bSuccess = (pCBack->Alloc)(hKXC,
                                 KXCALLOC_CONTEXT,
                                 TagLen,
                                 (PKXCVOID *)&pBuffer->pBuffer);
      if (bSuccess)
      {
        pBuffer->BufLen = TagLen;
      }
    }

    /****************************************************************/
    /* Take a copy of the buffer returned from earModifyMsgForEMS   */
    /****************************************************************/
    if (bSuccess)
    {
      if (TagLen > 0)
      {
        TRACE0 ("DoNotify: Copy tag buffer\n");
        memcpy(pBuffer->pBuffer, 
               pTag, 
               TagLen);
      }
      *pActions |= KXCEXCACT_NEWNOTIFYTAG;
    }
  }
  
  /******************************************************************/
  /* Finally, if earModifyForEMS allocated buffers free them        */
  /******************************************************************/
  if (pMsg != NULL)
  {
    TRACE0 ("DoNotify: free temp msg buffer\n");
    free (pMsg);
  }

  if (pTag != NULL)
  {
    TRACE0 ("DoNotify: free temp tag buffer\n");
    free (pTag);
  }

  TRACE1 ("DoNotify: Return %s\n", (bSuccess ? "success" : "failure"));
  return (bSuccess);
}


/********************************************************************/
/* Public functions, the interface to eBP                           */
/********************************************************************/
/********************************************************************/
/* kxcEarInit                                                       */
/*                                                                  */
/* Purpose    Initializes the error analysis routine.  All we do    */
/*            is allocate a KXCEXCACT structure against the context */
/*            to keep track of any buffers we allocate.             */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            pXCtl    - Our context                                */
/*            pConfig  - Initialization structure                   */
/*            InitType - The reason for initializing                */
/*                                                                  */
/* Returns                                                          */
/*            TRUE     - OK                                         */
/*            FALSE    - Alloc failed                               */
/*                                                                  */
/********************************************************************/
KXCBOOL kxcEARInit(PKXCXCTL   pXCtl,
                   PKXCCONFIG pConfig,
                   KXCULONG   InitType)
{
KXCBOOL         bSuccess = TRUE;
HKXC            hKXC;
PKXCEXCACT      pExcAct = NULL;
KXCCBACK        *pCBack;

  /******************************************************************/
  /* Pick up the context                                            */
  /******************************************************************/
  TRACE0 ("kxcEARInit: Initialising\n");
  hKXC = pXCtl->hKxc;
  pCBack = (KXCCBACK *)pXCtl->pCallBacks;

  /******************************************************************/
  /* Allocate a KXCEXCACT structure to store any new buffers in     */
  /******************************************************************/
  TRACE0 ("kxcEARInit: Alloc ExcAct\n");
  bSuccess = (pCBack->Alloc)(hKXC,
                             KXCALLOC_CONTEXT,
                             sizeof(KXCEXCACT),
                             (PKXCVOID *)&pExcAct);

  /******************************************************************/
  /* OK so far                                                      */
  /******************************************************************/
  if (bSuccess)
  {
    /****************************************************************/
    /* Initialise the context                                       */
    /****************************************************************/
    memset(pExcAct, 0, sizeof(KXCEXCACT));

    /****************************************************************/
    /* Set up the Context details                                   */
    /****************************************************************/
    pXCtl->pHandleContext = pExcAct;
  };

  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  TRACE1 ("kxcEARInit: Return %s\n", (bSuccess ? "success" : "failure"));
  return(bSuccess);
}

/********************************************************************/
/* kxcEarTerm                                                       */
/*                                                                  */
/* Purpose    Called when the eBP handle is terminated.  Free       */
/*            the Context we allocated and any buffers we might     */
/*            have hanging off this context.                        */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            pXCtl    - Our context                                */
/*            TermType - The reason for terminating                 */
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*                                                                  */
/********************************************************************/
KXCBOOL kxcEARTerm(PKXCXCTL pXCtl,
                   KXCULONG TermType)
{
KXCBOOL         bSuccess = TRUE;
HKXC            hKXC;
PKXCEXCACT      pExcAct = NULL;
KXCCBACK        *pCBack;

  /******************************************************************/
  /* Pick up the context                                            */
  /******************************************************************/
  hKXC    = pXCtl->hKxc;
  pCBack  = (KXCCBACK *)pXCtl->pCallBacks;
  pExcAct = (PKXCEXCACT)pXCtl->pHandleContext;

  TRACE0 ("kxcEARTerm: Terminating\n");

  /******************************************************************/
  /* If the HandleContext is good                                   */
  /******************************************************************/
  if (pExcAct != NULL)
  {
    /****************************************************************/
    /* Release the Notify message buffer                            */
    /****************************************************************/
    if (pExcAct->NotifyMsg.pBuffer != NULL)
    {
      TRACE0 ("kxcEARTerm: Free Notify Message\n");
      (pCBack->Free)(hKXC, pExcAct->NotifyMsg.pBuffer);
      pExcAct->NotifyMsg.pBuffer = NULL;
      pExcAct->NotifyMsg.BufLen  = 0L;
    };
    
    /****************************************************************/
    /* Release the Notify tag buffer                                */
    /****************************************************************/
    if (pExcAct->NotifyTag.pBuffer != NULL)
    {
      TRACE0 ("kxcEARTerm: Free Notify Tag\n");
      (pCBack->Free)(hKXC, pExcAct->NotifyTag.pBuffer);
      pExcAct->NotifyTag.pBuffer = NULL;
      pExcAct->NotifyTag.BufLen  = 0L;
    };
    
    /****************************************************************/
    /* Release the EMS message buffer                               */
    /****************************************************************/
    if (pExcAct->EMSMsg.pBuffer != NULL)
    {
      TRACE0 ("kxcEARTerm: Free EMS Message\n");
      (pCBack->Free)(hKXC, pExcAct->EMSMsg.pBuffer);
      pExcAct->EMSMsg.pBuffer = NULL;
      pExcAct->EMSMsg.BufLen  = 0L;
    };
    
    /****************************************************************/
    /* Release the EMS tag buffer                                   */
    /****************************************************************/
    if (pExcAct->EMSTag.pBuffer != NULL)
    {
      TRACE0 ("kxcEARTerm: Free EMS Tag\n");
      (pCBack->Free)(hKXC, pExcAct->EMSTag.pBuffer);
      pExcAct->EMSTag.pBuffer = NULL;
      pExcAct->EMSTag.BufLen  = 0L;
    };
    
    /****************************************************************/
    /* Release the RAW message buffer                               */
    /****************************************************************/
    if (pExcAct->RawMessage.pBuffer != NULL)
    {
      TRACE0 ("kxcEARTerm: Free RAW Message\n");
      (pCBack->Free)(hKXC, pExcAct->RawMessage.pBuffer);
      pExcAct->RawMessage.pBuffer = NULL;
      pExcAct->RawMessage.BufLen  = 0L;
    };
    
    /****************************************************************/
    /* Release the exception structure                              */
    /****************************************************************/
    if (pExcAct->pException != NULL)
    {
      TRACE0 ("kxcEARTerm: Free Exception\n");
      (pCBack->Free)(hKXC, pExcAct->pException);
      pExcAct->pException = NULL;
    };
    
    /****************************************************************/
    /* Release the error profile structure                          */
    /****************************************************************/
    if (pExcAct->pErrProfile != NULL)
    {
      TRACE0 ("kxcEARTerm: Free ErrProfile\n");
      (pCBack->Free)(hKXC, pExcAct->pErrProfile);
      pExcAct->pErrProfile = NULL;
    };
    
    /****************************************************************/
    /* Release the user log data (and it's buffers)                 */
    /****************************************************************/
    if (pExcAct->pUserLogData != NULL)
    {
      TRACE0 ("kxcEARTerm: Free UserLogData\n");
      if (pExcAct->pUserLogData->pUserText != NULL)
      {
        (pCBack->Free)(hKXC, pExcAct->pUserLogData->pUserText);
        pExcAct->pUserLogData->pUserText = NULL;
      }
      
      if (pExcAct->pUserLogData->DataBuf.pBuffer != NULL)
      {
        (pCBack->Free)(hKXC, pExcAct->pUserLogData->DataBuf.pBuffer);
        pExcAct->pUserLogData->DataBuf.pBuffer = NULL;
        pExcAct->pUserLogData->DataBuf.BufLen = 0L;
      }
      
      (pCBack->Free)(hKXC, pExcAct->pUserLogData);
      pExcAct->pUserLogData = NULL;
    };
    
    /****************************************************************/
    /* Release the alert data (and it's buffers)                    */
    /****************************************************************/
    if (pExcAct->pAlertData != NULL)
    {
      TRACE0 ("kxcEARTerm: Free Alertdata\n");    
      if (pExcAct->pAlertData->pUserText != NULL)
      {
        (pCBack->Free)(hKXC, pExcAct->pAlertData->pUserText);
        pExcAct->pAlertData->pUserText = NULL;
      }
      
      if (pExcAct->pAlertData->DataBuf.pBuffer != NULL)
      {
        (pCBack->Free)(hKXC, pExcAct->pAlertData->DataBuf.pBuffer);
        pExcAct->pAlertData->DataBuf.pBuffer = NULL;
        pExcAct->pAlertData->DataBuf.BufLen = 0L;
      }
      
      (pCBack->Free)(hKXC, pExcAct->pAlertData);
      pExcAct->pAlertData = NULL;
    };
    
    /****************************************************************/
    /* Release the context                                          */
    /****************************************************************/
    TRACE0 ("kxcEARTerm: Free ExcAct\n");
    (pCBack->Free)(hKXC, pExcAct);
    pExcAct = NULL;
    pXCtl->pHandleContext = NULL;
  };
  
  /******************************************************************/
  /* Return to the caller                                           */
  /******************************************************************/
  TRACE1 ("kxcEARTerm: Return %s\n", (bSuccess ? "success" : "failure"));
  return(bSuccess);
}

/********************************************************************/
/* kxcEarAnalyse                                                    */
/*                                                                  */
/* Purpose    Called by eBP when an error occurs.  This             */
/*            routine looks at the Actions field in the error       */
/*            profile structure and farms out the work accordingly. */
/*                                                                  */
/* Notes      Any buffers allocated and passed back to eBP are      */
/*            remembered in our context and freed on termination.   */
/*            In this example if Actions is NONE (default) all      */
/*            actions are performed.                                */
/*                                                                  */
/* Parameters                                                       */
/*          Input                                                   */
/*            pXCtl    - Our context                                */
/*            pExcDtl  - The details of the Exception               */
/*            pAction  - The actions taken by the EAR               */
/*                                                                  */
/* Returns                                                          */
/*            TRUE                                                  */
/*            FALSE - One of the servicing routines failed.         */
/*                                                                  */
/********************************************************************/
KXCBOOL kxcEARAnalyse(PKXCXCTL   pXCtl,
                      PKXCEXCDTL pExcDtl,
                      PKXCEXCACT pAction)
{
KXCBOOL         bSuccess = TRUE;
HKXC            hKXC;
PKXCEXCACT      pExcAct = NULL;
KXCCBACK        *pCBack;
KXCULONG        Actions;

  TRACE0 ("kxcEARAnalyse: Entering\n");

  /******************************************************************/
  /* Pick up the context                                            */
  /******************************************************************/
  hKXC    = pXCtl->hKxc;
  pCBack  = (KXCCBACK *)pXCtl->pCallBacks;
  pExcAct = (PKXCEXCACT)pXCtl->pHandleContext;

  /******************************************************************/
  /* Initialize the actions flag                                    */
  /******************************************************************/
  pAction->Actions = KXCSERVACT_NONE;

  TRACE1 ("ErrorProfileData Actions is %d\n", 
          pExcDtl->pErrorProfileData->Actions);

  /******************************************************************/
  /* If we should do anything                                       */
  /******************************************************************/
  if (!(pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_NOACTION))
  {
    TRACE0 ("kxcEARAnalyse: Action wasn't NOACTION \n");
    /****************************************************************/
    /* Is it a notify?                                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_NOTIFY))
    {
      TRACE0 ("kxcEARAnalyse: Action Notify\n");
      bSuccess = DoNotify(hKXC, pCBack, pExcDtl,
                          pExcAct,
                          &Actions);

      if (bSuccess)
      {
        TRACE0 ("kxcEARAnalyse: Notify OK\n");
        /************************************************************/
        /* Copy the modified details depending on Actions           */
        /************************************************************/
        if (Actions & KXCEXCACT_NEWNOTIFYMSG)
        {
          TRACE0 ("kxcEARAnalyse: New Notify Msg Buffer\n");
          pAction->NotifyMsg  = pExcAct->NotifyMsg;
          pAction->Actions   |= KXCEXCACT_NEWNOTIFYMSG;
        }
        if (Actions & KXCEXCACT_NEWNOTIFYMSGLEN)
        {
          TRACE0 ("kxcEARAnalyse: New Notify Msg Len\n");
          pAction->NotifyMsgLen  = pExcAct->NotifyMsgLen;
          pAction->Actions      |= KXCEXCACT_NEWNOTIFYMSGLEN;
        }
        if (Actions & KXCEXCACT_NEWNOTIFYTAG)
        {
          TRACE0 ("kxcEARAnalyse: New Notify Tag Buffer\n");
          pAction->NotifyTag  = pExcAct->NotifyTag;
          pAction->Actions   |= KXCEXCACT_NEWNOTIFYTAG;
        }
        if (Actions & KXCEXCACT_NEWNOTIFYTAGLEN)
        {
          TRACE0 ("kxcEARAnalyse: New Notify Tag Length\n");
          pAction->NotifyTagLen  = pExcAct->NotifyTagLen;
          pAction->Actions      |= KXCEXCACT_NEWNOTIFYTAGLEN;
        }
      }
    }

    /****************************************************************/
    /* Is it a EMS?                                                 */
    /****************************************************************/
    if ((bSuccess) &&
        (pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_EMS))
    {
      TRACE0 ("kxcEARAnalyse: Action EMS\n");
      bSuccess = DoEMS(hKXC, pCBack, pExcDtl,
                       pExcAct,
                       &Actions);

      if (bSuccess)
      {
        TRACE0 ("kxcEARAnalyse: EMS OK\n");
        /************************************************************/
        /* Copy the modified details depending on Actions           */
        /************************************************************/
        if (Actions & KXCEXCACT_NEWEMSMSG)
        {
          TRACE0 ("kxcEARAnalyse: New EMS Msg Buffer\n");
          pAction->EMSMsg   = pExcAct->EMSMsg;
          pAction->Actions |= KXCEXCACT_NEWEMSMSG;
        }
        if (Actions & KXCEXCACT_NEWEMSMSGLEN)
        {
          TRACE0 ("kxcEARAnalyse: New EMS Msg Len\n");
          pAction->EMSMsgLen  = pExcAct->EMSMsgLen;
          pAction->Actions   |= KXCEXCACT_NEWEMSMSGLEN;
        }
        if (Actions & KXCEXCACT_NEWEMSTAG)
        {
          TRACE0 ("kxcEARAnalyse: New EMS Tag Buffer\n");
          pAction->EMSTag   = pExcAct->EMSTag;
          pAction->Actions |= KXCEXCACT_NEWEMSTAG;
        }
        if (Actions & KXCEXCACT_NEWEMSTAGLEN)
        {
          TRACE0 ("kxcEARAnalyse: New EMS Tag Length\n");
          pAction->EMSTagLen  = pExcAct->EMSTagLen;
          pAction->Actions   |= KXCEXCACT_NEWEMSTAGLEN;
        }
        if (Actions & KXCEXCACT_NEWEMSRAW)
        {
          TRACE0 ("kxcEARAnalyse: New EMS RAW Buffer\n");
          pAction->RawMessage  = pExcAct->RawMessage;
          pAction->Actions    |= KXCEXCACT_NEWEMSRAW;
        }
        if (Actions & KXCEXCACT_NEWEMSRAWLEN)
        {
          TRACE0 ("kxcEARAnalyse: New EMS Raw Msg Len\n");
          pAction->RawMsgLen  = pExcAct->RawMsgLen;
          pAction->Actions   |= KXCEXCACT_NEWEMSRAWLEN;
        }
      }
    }

    /****************************************************************/
    /* Is it an Alert?                                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_ALERT))
    {
      TRACE0 ("kxcEARAnalyse: Action Alert\n");
      /**************************************************************/
      /* Call the function to generate the Alert Message            */
      /**************************************************************/
      bSuccess = DoAlert(hKXC, pCBack, pExcDtl, 
                         &pExcAct->pAlertData);

      /**************************************************************/
      /* Copy the alert data pointer into the output structure      */
      /**************************************************************/
      if (bSuccess)
      {
        TRACE0 ("kxcEARAnalyse: Alert data set\n");
        pAction->pAlertData = pExcAct->pAlertData;
        pAction->Actions |= KXCEXCACT_ALERTDATA;
      }
    }

    /****************************************************************/
    /* Is it a Log?                                                 */
    /****************************************************************/
    if ((bSuccess) &&
        (pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_LOG))
    {
      TRACE0 ("kxcEARAnalyse: Action Log\n");
      bSuccess = DoLog(hKXC, pCBack, pExcDtl, 
                       &pExcAct->pUserLogData);

      /**************************************************************/
      /* Copy the alert data pointer into the output structure      */
      /**************************************************************/
      if (bSuccess)
      {
        TRACE0 ("kxcEARAnalyse: Log data set\n");
        pAction->pUserLogData = pExcAct->pUserLogData;
        pAction->Actions |= KXCEXCACT_LOGDATA;
      }
    }

    /****************************************************************/
    /* Is it a Reset?                                               */
    /****************************************************************/
    if ((bSuccess) &&
        (pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_RESET))
    {
      TRACE0 ("kxcEARAnalyse: Reset\n");
      bSuccess = DoReset(hKXC, pCBack, pExcDtl,
                         &pExcAct->pErrProfile,
                         &pExcAct->pException,
                         &Actions);

      if (bSuccess)
      {
        TRACE0 ("kxcEARAnalyse: DoReset OK\n");
        /************************************************************/
        /* Copy the modified details depending on Actions           */
        /************************************************************/
        if (Actions & KXCEXCACT_NEWEXCEPTION)
        {
          TRACE0 ("kxcEARAnalyse: New Exception\n");
          pAction->pException = pExcAct->pException;
          pAction->Actions |= KXCEXCACT_NEWEXCEPTION;
        }
        if (Actions & KXCEXCACT_NEWERRPROFILE)
        {
          TRACE0 ("kxcEARAnalyse: New Profile\n");
          pAction->pErrProfile = pExcAct->pErrProfile;
          pAction->Actions |= KXCEXCACT_NEWERRPROFILE;
        }
      }
    }

    /****************************************************************/
    /* Is it a EMSRaw?                                              */
    /****************************************************************/
    if ((bSuccess) &&
        (pExcDtl->pErrorProfileData->Actions & KXCEP_ACTION_EMSRAW))
    {
      TRACE0 ("kxcEARAnalyse: Action EMSRaw\n");
      bSuccess = DoEMSRaw(hKXC, pCBack, pExcDtl,
                          pExcAct,
                          &Actions);

      if (bSuccess)
      {
        TRACE0 ("kxcEARAnalyse: EMSRaw OK\n");
        /************************************************************/
        /* Copy the modified details depending on Actions           */
        /************************************************************/
        if (Actions & KXCEXCACT_NEWEMSRAW)
        {
          TRACE0 ("kxcEARAnalyse: New EMS RAW Buffer\n");
          pAction->RawMessage  = pExcAct->RawMessage;
          pAction->Actions    |= KXCEXCACT_NEWEMSRAW;
        }
        if (Actions & KXCEXCACT_NEWEMSRAWLEN)
        {
          TRACE0 ("kxcEARAnalyse: New EMS Raw Msg Len\n");
          pAction->RawMsgLen  = pExcAct->RawMsgLen;
          pAction->Actions   |= KXCEXCACT_NEWEMSMSGLEN;
        }
      }
    }
  }

  TRACE1 ("kxcEARAnalyse: Return %s\n", 
          (bSuccess ? "success" : "failure"));
  return (bSuccess);
}

/********************************************************************/
/* End of module kxcear2.c. Change History follows                  */
/********************************************************************/
/*
 * $Log: kxcseara.c,v $
 * Revision 1.4  2005/09/23 14:59:57  e_vvb
 * DoNotify call can cause exception in the customer's ILS
 *
 * Revision 1.3  2001/12/21 16:12:56  mboag
 * XC320 Merge of "kxc310_Dist" tagged "kxc310_Dist_lastmerge2trunk" to bring CASP
 *       and 310 maintenance into 320 trunk
 *
 *
 * Revision 1.2  2001/05/17 15:32:25  pross
 * Merge back from CTVC to CVS main stream.
 *
 * Revision 1.1.1.1  2001/05/16 09:49:27  pross
 * Created from CTVC xcv310 010511-F1
 *
// 
//    Rev 1.1.1.0   05 Mar 2001 16:08:02   RHEND
// XCV320: 1-4CUU8 Removed references to Roma
// 
//    Rev 1.1   16 Mar 2000 11:56:40   MMALC
// XC300: I671513 Correct build instruction comment
// 
//    Rev 1.0   19 Feb 1999 18:05:06   GATAY
// XC300: Initial Upload for BSP3.0
 */
/********************************************************************/
